@@ -15,34 +15,20 @@ from pathlib import Path
 from typing import Type
 
 from murfey.client.context import Context
+from murfey.client.contexts.atlas import AtlasContext
 from murfey.client.contexts.clem import CLEMContext
-from murfey.client.contexts.spa import SPAContext, SPAModularContext
+from murfey.client.contexts.spa import SPAModularContext
 from murfey.client.contexts.spa_metadata import SPAMetadataContext
 from murfey.client.contexts.tomo import TomographyContext
+from murfey.client.contexts.tomo_metadata import TomographyMetadataContext
 from murfey.client.instance_environment import MurfeyInstanceEnvironment
 from murfey.client.rsync import RSyncerUpdate, TransferResult
-from murfey.client.tui.forms import FormDependency
-from murfey.util import Observer, get_machine_config_client
-from murfey.util.models import PreprocessingParametersTomo, ProcessingParametersSPA
+from murfey.util.client import Observer, get_machine_config_client
+from murfey.util.mdoc import get_block
+from murfey.util.models import ProcessingParametersSPA, ProcessingParametersTomo
 
 logger = logging.getLogger("murfey.client.analyser")
 
-spa_form_dependencies: dict = {
-    "use_cryolo": FormDependency(
-        dependencies={"estimate_particle_diameter": False}, trigger_value=False
-    ),
-    "estimate_particle_diameter": FormDependency(
-        dependencies={
-            "use_cryolo": True,
-            "boxsize": "None",
-            "small_boxsize": "None",
-            "mask_diameter": "None",
-            "particle_diameter": "None",
-        },
-        trigger_value=True,
-    ),
-}
-
 
 class Analyser(Observer):
     def __init__(
@@ -64,7 +50,7 @@ class Analyser(Observer):
         self._environment = environment
         self._force_mdoc_metadata = force_mdoc_metadata
         self.parameters_model: (
-            Type[ProcessingParametersSPA] | Type[PreprocessingParametersTomo] | None
+            Type[ProcessingParametersSPA] | Type[ProcessingParametersTomo] | None
         ) = None
 
         self.queue: queue.Queue = queue.Queue()
@@ -81,7 +67,10 @@ class Analyser(Observer):
             else {}
         )
 
-    def _find_extension(self, file_path: Path):
+    def __repr__(self) -> str:
+        return f"<Analyser ({self._basepath})>"
+
+    def _find_extension(self, file_path: Path) -> bool:
         """
         Identifies the file extension and stores that information in the class.
         """
@@ -97,25 +86,30 @@ class Analyser(Observer):
             .get(file_path.suffix)
         ):
             if not any(r in file_path.name for r in required_substrings):
-                return []
+                return False
 
-        # Checks for MRC, TIFF, TIF, and EER files if no extension has been defined
-        if (
-            file_path.suffix in (".mrc", ".tiff", ".tif", ".eer")
-            and not self._extension
-        ):
-            logger.info(f"File extension determined: {file_path.suffix}")
-            self._extension = file_path.suffix
-        # Check for TIFF, TIF, or EER if the file's already been assigned an extension
-        elif (
-            file_path.suffix in (".tiff", ".tif", ".eer")
-            and self._extension != file_path.suffix
-        ):
-            logger.info(f"File extension re-evaluated: {file_path.suffix}")
-            self._extension = file_path.suffix
+        # Checks for MRC, TIFF, TIF, and EER files
+        if file_path.suffix in (".mrc", ".tiff", ".tif", ".eer"):
+            if not self._extension:
+                logger.info(f"File extension determined: {file_path.suffix}")
+                self._extension = file_path.suffix
+            elif self._extension != file_path.suffix:
+                logger.info(f"File extension re-evaluated: {file_path.suffix}")
+                self._extension = file_path.suffix
+            return True
+        # If we see an .mdoc file first, use that to determine the file extensions
+        elif file_path.suffix == ".mdoc":
+            with open(file_path, "r") as md:
+                md.seek(0)
+                mdoc_data_block = get_block(md)
+            if subframe_path := mdoc_data_block.get("SubFramePath"):
+                self._extension = Path(subframe_path).suffix
+                return True
         # Check for LIF files separately
         elif file_path.suffix == ".lif":
             self._extension = file_path.suffix
+            return True
+        return False
 
     def _find_context(self, file_path: Path) -> bool:
         """
@@ -124,9 +118,7 @@ class Analyser(Observer):
         stages of processing. Actions to take for individual files will be determined
         in the Context classes themselves.
         """
-        if "atlas" in file_path.parts:
-            self._context = SPAMetadataContext("epu", self._basepath)
-            return True
+        logger.debug(f"Finding context using file {str(file_path)!r}")
 
         # CLEM workflow checks
         # Look for LIF and XLIF files
@@ -135,86 +127,59 @@ class Analyser(Observer):
             return True
         # Look for TIFF files associated with CLEM workflow
         # Leica's autosave mode seems to name the TIFFs in the format
-        # PostionXX--ZXX-CXX.tif
-        if (
-            "--" in file_path.name
-            and file_path.suffix in (".tiff", ".tif")
-            and self._environment
-        ):
-            created_directories = set(
-                get_machine_config_client(
-                    str(self._environment.url.geturl()),
-                    instrument_name=self._environment.instrument_name,
-                    demo=self._environment.demo,
-                ).get("analyse_created_directories", [])
-            )
-            if created_directories.intersection(set(file_path.parts)):
-                self._context = CLEMContext("leica", self._basepath)
-                return True
+        # PostionXX--ZXX--CXX.tif
+        if all(
+            pattern in file_path.name for pattern in ("--Z", "--C")
+        ) and file_path.suffix in (".tiff", ".tif"):
+            self._context = CLEMContext("leica", self._basepath)
+            return True
 
         # Tomography and SPA workflow checks
-        split_file_name = file_path.name.split("_")
-        if split_file_name:
+        if "atlas" in file_path.parts:
+            self._context = AtlasContext("epu", self._basepath)
+            return True
+
+        if "Metadata" in file_path.parts or file_path.name == "EpuSession.dm":
+            self._context = SPAMetadataContext("epu", self._basepath)
+            return True
+        elif (
+            "Batch" in file_path.parts
+            or "SearchMaps" in file_path.parts
+            or "Thumbnails" in file_path.parts
+            or file_path.name == "Session.dm"
+        ):
+            self._context = TomographyMetadataContext("tomo", self._basepath)
+            return True
+
+        split_file_stem = file_path.stem.split("_")
+        if split_file_stem:
+            if split_file_stem[-1] == "gain":
+                return False
+
             # Files starting with "FoilHole" belong to the SPA workflow
-            if split_file_name[0].startswith("FoilHole"):
+            if split_file_stem[0].startswith("FoilHole") and split_file_stem[-1] in [
+                "Fractions",
+                "fractions",
+                "EER",
+            ]:
                 if not self._context:
                     logger.info("Acquisition software: EPU")
-                    if self._environment:
-                        try:
-                            cfg = get_machine_config_client(
-                                str(self._environment.url.geturl()),
-                                instrument_name=self._environment.instrument_name,
-                                demo=self._environment.demo,
-                            )
-                        except Exception as e:
-                            logger.error(f"Exception encountered: {e}")
-                            cfg = {}
-                    else:
-                        cfg = {}
-                    self._context = (
-                        SPAModularContext("epu", self._basepath)
-                        if cfg.get("modular_spa")
-                        else SPAContext("epu", self._basepath)
-                    )
+                    self._context = SPAModularContext("epu", self._basepath)
                 self.parameters_model = ProcessingParametersSPA
                 return True
 
             # Files starting with "Position" belong to the standard tomography workflow
+            # NOTE: not completely reliable, mdocs can be in tomography metadata as well
             if (
-                split_file_name[0] == "Position"
+                split_file_stem[0] == "Position"
                 or "[" in file_path.name
-                or "Fractions" in split_file_name[-1]
-                or "fractions" in split_file_name[-1]
-                or "EER" in split_file_name[-1]
+                or split_file_stem[-1] in ["Fractions", "fractions", "EER"]
+                or file_path.suffix == ".mdoc"
             ):
                 if not self._context:
                     logger.info("Acquisition software: tomo")
                     self._context = TomographyContext("tomo", self._basepath)
-                    self.parameters_model = PreprocessingParametersTomo
-                return True
-
-            # Files with these suffixes belong to the serial EM tomography workflow
-            if file_path.suffix in (".mrc", ".tiff", ".tif", ".eer"):
-                # Ignore batch files and search maps
-                if any(p in file_path.parts for p in ("Batch", "SearchMaps")):
-                    return False
-                # Ignore JPG files
-                if file_path.with_suffix(".jpg").is_file():
-                    return False
-                # Ignore the averaged movies written out by the Falcon
-                if (
-                    len(
-                        list(
-                            file_path.parent.glob(
-                                f"{file_path.name}*{file_path.suffix}"
-                            )
-                        )
-                    )
-                    > 1
-                ):
-                    return False
-                self._context = TomographyContext("serialem", self._basepath)
-                self.parameters_model = PreprocessingParametersTomo
+                    self.parameters_model = ProcessingParametersTomo
                 return True
         return False
 
@@ -225,7 +190,9 @@ class Analyser(Observer):
                     transferred_file, environment=self._environment
                 )
         except Exception as e:
-            logger.error(f"An exception was encountered post transfer: {e}")
+            logger.error(
+                f"An exception was encountered post transfer: {e}", exc_info=True
+            )
 
     def _analyse(self):
         logger.info("Analyser thread started")
@@ -247,6 +214,13 @@ class Analyser(Observer):
                     and not self._context
                 ):
                     self._context = SPAMetadataContext("epu", self._basepath)
+                elif (
+                    "Batch" in transferred_file.parts
+                    or "SearchMaps" in transferred_file.parts
+                    or transferred_file.name == "Session.dm"
+                    and not self._context
+                ):
+                    self._context = TomographyMetadataContext("tomo", self._basepath)
                 self.post_transfer(transferred_file)
             else:
                 dc_metadata = {}
@@ -271,12 +245,15 @@ class Analyser(Observer):
                     elif transferred_file.suffix == ".mdoc":
                         mdoc_for_reading = transferred_file
                 if not self._context:
-                    self._find_extension(transferred_file)
+                    valid_extension = self._find_extension(transferred_file)
+                    if not valid_extension:
+                        logger.error(f"No extension found for {transferred_file}")
+                        continue
                     found = self._find_context(transferred_file)
                     if not found:
-                        # logger.warning(
-                        #     f"Context not understood for {transferred_file}, stopping analysis"
-                        # )
+                        logger.debug(
+                            f"Couldn't find context for {str(transferred_file)!r}"
+                        )
                         self.queue.task_done()
                         continue
                     elif self._extension:
@@ -290,16 +267,11 @@ class Analyser(Observer):
                             )
                         except Exception as e:
                             logger.error(f"Exception encountered: {e}")
-                        if "atlas" not in transferred_file.parts:
+                        if not isinstance(self._context, AtlasContext):
                             if not dc_metadata:
                                 try:
                                     dc_metadata = self._context.gather_metadata(
-                                        (
-                                            transferred_file.with_suffix(".mdoc")
-                                            if self._context._acquisition_software
-                                            == "serialem"
-                                            else self._xml_file(transferred_file)
-                                        ),
+                                        self._xml_file(transferred_file),
                                         environment=self._environment,
                                     )
                                 except NotImplementedError:
@@ -309,6 +281,10 @@ class Analyser(Observer):
                                         f"Metadata gathering failed with a key error for key: {e.args[0]}"
                                     )
                                     raise e
+                                except ValueError as e:
+                                    logger.error(
+                                        f"Metadata gathering failed with a value error: {e}"
+                                    )
                             if not dc_metadata or not self._force_mdoc_metadata:
                                 self._unseen_xml.append(transferred_file)
                             else:
@@ -323,14 +299,6 @@ class Analyser(Observer):
                                 self.notify(
                                     {
                                         "form": dc_metadata,
-                                        "dependencies": (
-                                            spa_form_dependencies
-                                            if isinstance(self._context, SPAContext)
-                                            or isinstance(
-                                                self._context, SPAModularContext
-                                            )
-                                            else {}
-                                        ),
                                     }
                                 )
 
@@ -341,9 +309,16 @@ class Analyser(Observer):
                     )
                     self.post_transfer(transferred_file)
 
+                elif isinstance(self._context, AtlasContext):
+                    logger.debug(f"File {transferred_file.name!r} is part of the atlas")
+                    self.post_transfer(transferred_file)
+
                 # Handle files with tomography and SPA context differently
                 elif not self._extension or self._unseen_xml:
-                    self._find_extension(transferred_file)
+                    valid_extension = self._find_extension(transferred_file)
+                    if not valid_extension:
+                        logger.error(f"No extension found for {transferred_file}")
+                        continue
                     if self._extension:
                         logger.info(
                             f"Extension found successfully for {transferred_file}"
@@ -355,54 +330,52 @@ class Analyser(Observer):
                             )
                         except Exception as e:
                             logger.error(f"Exception encountered: {e}")
-                        if "atlas" not in transferred_file.parts:
-                            if not dc_metadata:
-                                try:
-                                    dc_metadata = self._context.gather_metadata(
-                                        mdoc_for_reading
-                                        or self._xml_file(transferred_file),
-                                        environment=self._environment,
-                                    )
-                                except KeyError as e:
-                                    logger.error(
-                                        f"Metadata gathering failed with a key error for key: {e.args[0]}"
-                                    )
-                                    raise e
-                            if not dc_metadata or not self._force_mdoc_metadata:
-                                mdoc_for_reading = None
-                                self._unseen_xml.append(transferred_file)
-                            if dc_metadata:
-                                self._unseen_xml = []
-                                if dc_metadata.get("file_extension"):
-                                    self._extension = dc_metadata["file_extension"]
-                                else:
-                                    dc_metadata["file_extension"] = self._extension
-                                dc_metadata["acquisition_software"] = (
-                                    self._context._acquisition_software
+                        if not dc_metadata:
+                            try:
+                                dc_metadata = self._context.gather_metadata(
+                                    mdoc_for_reading
+                                    or self._xml_file(transferred_file),
+                                    environment=self._environment,
                                 )
-                                self.notify(
-                                    {
-                                        "form": dc_metadata,
-                                        "dependencies": (
-                                            spa_form_dependencies
-                                            if isinstance(self._context, SPAContext)
-                                            or isinstance(
-                                                self._context, SPAModularContext
-                                            )
-                                            else {}
-                                        ),
-                                    }
+                            except KeyError as e:
+                                logger.error(
+                                    f"Metadata gathering failed with a key error for key: {e.args[0]}"
                                 )
+                                raise e
+                        if not dc_metadata or not self._force_mdoc_metadata:
+                            mdoc_for_reading = None
+                            self._unseen_xml.append(transferred_file)
+                        if dc_metadata:
+                            self._unseen_xml = []
+                            if dc_metadata.get("file_extension"):
+                                self._extension = dc_metadata["file_extension"]
+                            else:
+                                dc_metadata["file_extension"] = self._extension
+                            dc_metadata["acquisition_software"] = (
+                                self._context._acquisition_software
+                            )
+                            self.notify(
+                                {
+                                    "form": dc_metadata,
+                                }
+                            )
                 elif isinstance(
                     self._context,
                     (
-                        TomographyContext,
                         SPAModularContext,
                         SPAMetadataContext,
+                        TomographyContext,
+                        TomographyMetadataContext,
                     ),
                 ):
+                    context = str(self._context).split(" ")[0].split(".")[-1]
+                    logger.debug(
+                        f"Transferring file {str(transferred_file)} with context {context!r}"
+                    )
                     self.post_transfer(transferred_file)
             self.queue.task_done()
+        logger.debug("Analyer thread has stopped analysing incoming files")
+        self.notify(final=True)
 
     def _xml_file(self, data_file: Path) -> Path:
         if not self._environment:
@@ -411,8 +384,8 @@ class Analyser(Observer):
         data_directories = self._murfey_config.get("data_directories", [])
         for dd in data_directories:
             if str(data_file).startswith(dd):
-                base_dir = Path(dd)
-                mid_dir = data_file.relative_to(dd).parent
+                base_dir = Path(dd).absolute()
+                mid_dir = data_file.relative_to(base_dir).parent
                 break
         else:
             return data_file.with_suffix(".xml")
@@ -420,7 +393,7 @@ class Analyser(Observer):
 
     def enqueue(self, rsyncer: RSyncerUpdate):
         if not self._stopping and rsyncer.outcome == TransferResult.SUCCESS:
-            absolute_path = (self._basepath / rsyncer.file_path).resolve()
+            absolute_path = (self._basepath / rsyncer.file_path).absolute()
             self.queue.put(absolute_path)
 
     def start(self):
@@ -431,6 +404,16 @@ class Analyser(Observer):
         logger.info(f"Analyser thread starting for {self}")
         self.thread.start()
 
+    def request_stop(self):
+        self._stopping = True
+        self._halt_thread = True
+
+    def is_safe_to_stop(self):
+        """
+        Checks that the analyser thread is safe to stop
+        """
+        return self._stopping and self._halt_thread and not self.queue.qsize()
+
     def stop(self):
         logger.debug("Analyser thread stop requested")
         self._stopping = True
@@ -440,5 +423,8 @@ class Analyser(Observer):
                 self.queue.put(None)
                 self.thread.join()
         except Exception as e:
-            logger.error(f"Exception encountered while stopping analyser: {e}")
+            logger.error(
+                f"Exception encountered while stopping Analyser: {e}",
+                exc_info=True,
+            )
         logger.debug("Analyser thread stop completed")
