@@ -1,12 +1,10 @@
 from __future__ import annotations
 
 import logging
-from datetime import datetime
 from itertools import count
 from pathlib import Path
-from typing import Any, Dict, List, NamedTuple, Optional, OrderedDict, Tuple
+from typing import Any, Dict, List, Optional, OrderedDict, Tuple
 
-import requests
 import xmltodict
 
 from murfey.client.context import Context, ProcessingParameter
@@ -15,118 +13,17 @@ from murfey.client.instance_environment import (
     MurfeyID,
     MurfeyInstanceEnvironment,
 )
-from murfey.util import (
-    authorised_requests,
-    capture_get,
-    capture_post,
-    get_machine_config_client,
+from murfey.util.client import capture_get, capture_post, get_machine_config_client
+from murfey.util.spa_metadata import (
+    foil_hole_data,
+    foil_hole_from_file,
+    get_grid_square_atlas_positions,
+    grid_square_data,
+    grid_square_from_file,
 )
 
 logger = logging.getLogger("murfey.client.contexts.spa")
 
-requests.get, requests.post, requests.put, requests.delete = authorised_requests()
-
-
-class FoilHole(NamedTuple):
-    session_id: int
-    id: int
-    grid_square_id: int
-    x_location: Optional[float] = None
-    y_location: Optional[float] = None
-    x_stage_position: Optional[float] = None
-    y_stage_position: Optional[float] = None
-    readout_area_x: Optional[int] = None
-    readout_area_y: Optional[int] = None
-    thumbnail_size_x: Optional[int] = None
-    thumbnail_size_y: Optional[int] = None
-    pixel_size: Optional[float] = None
-    image: str = ""
-    diameter: Optional[float] = None
-
-
-class GridSquare(NamedTuple):
-    session_id: int
-    id: int
-    x_location: Optional[float] = None
-    y_location: Optional[float] = None
-    x_stage_position: Optional[float] = None
-    y_stage_position: Optional[float] = None
-    readout_area_x: Optional[int] = None
-    readout_area_y: Optional[int] = None
-    thumbnail_size_x: Optional[int] = None
-    thumbnail_size_y: Optional[int] = None
-    pixel_size: Optional[float] = None
-    image: str = ""
-    tag: str = ""
-
-
-def _get_grid_square_atlas_positions(xml_path: Path, grid_square: str = "") -> Dict[
-    str,
-    Tuple[
-        Optional[int],
-        Optional[int],
-        Optional[float],
-        Optional[float],
-        Optional[int],
-        Optional[int],
-        Optional[float],
-    ],
-]:
-    with open(
-        xml_path,
-        "r",
-    ) as dm:
-        atlas_data = xmltodict.parse(dm.read())
-    tile_info = atlas_data["AtlasSessionXml"]["Atlas"]["TilesEfficient"]["_items"][
-        "TileXml"
-    ]
-    gs_pix_positions: Dict[
-        str,
-        Tuple[
-            Optional[int],
-            Optional[int],
-            Optional[float],
-            Optional[float],
-            Optional[int],
-            Optional[int],
-            Optional[float],
-        ],
-    ] = {}
-    for ti in tile_info:
-        try:
-            nodes = ti["Nodes"]["KeyValuePairs"]
-        except KeyError:
-            continue
-        required_key = ""
-        for key in nodes.keys():
-            if key.startswith("KeyValuePairOfintNodeXml"):
-                required_key = key
-                break
-        if not required_key:
-            continue
-        for gs in nodes[required_key]:
-            if not isinstance(gs, dict):
-                continue
-            if not grid_square or gs["key"] == grid_square:
-                gs_pix_positions[gs["key"]] = (
-                    int(float(gs["value"]["b:PositionOnTheAtlas"]["c:Center"]["d:x"])),
-                    int(float(gs["value"]["b:PositionOnTheAtlas"]["c:Center"]["d:y"])),
-                    float(gs["value"]["b:PositionOnTheAtlas"]["c:Physical"]["d:x"])
-                    * 1e9,
-                    float(gs["value"]["b:PositionOnTheAtlas"]["c:Physical"]["d:y"])
-                    * 1e9,
-                    int(
-                        float(gs["value"]["b:PositionOnTheAtlas"]["c:Size"]["d:width"])
-                    ),
-                    int(
-                        float(gs["value"]["b:PositionOnTheAtlas"]["c:Size"]["d:height"])
-                    ),
-                    float(gs["value"]["b:PositionOnTheAtlas"]["c:Rotation"]),
-                )
-                if grid_square:
-                    break
-    return gs_pix_positions
-
 
 def _file_transferred_to(
     environment: MurfeyInstanceEnvironment, source: Path, file_path: Path
@@ -140,7 +37,7 @@ def _file_transferred_to(
         return (
             Path(machine_config.get("rsync_basepath", ""))
             / Path(environment.default_destinations[source])
-            / file_path.relative_to(source)
+            / file_path.relative_to(source)  # need to strip out the rsync_module name
         )
     return (
         Path(machine_config.get("rsync_basepath", ""))
@@ -150,132 +47,42 @@ def _file_transferred_to(
     )
 
 
-def _grid_square_from_file(f: Path) -> int:
-    for p in f.parts:
-        if p.startswith("GridSquare"):
-            return int(p.split("_")[1])
-    raise ValueError(f"Grid square ID could not be determined from path {f}")
-
-
-def _foil_hole_from_file(f: Path) -> int:
-    return int(f.name.split("_")[1])
-
-
 def _grid_square_metadata_file(
     f: Path, data_directories: List[Path], visit: str, grid_square: int
 ) -> Path:
     for dd in data_directories:
         if str(f).startswith(str(dd)):
-            base_dir = dd
-            mid_dir = f.relative_to(dd).parent
+            base_dir = dd.absolute()
+            mid_dir = f.relative_to(base_dir).parent
             break
     else:
         raise ValueError(f"Could not determine grid square metadata path for {f}")
-    return (
+    metadata_file = (
         base_dir
         / visit
         / mid_dir.parent.parent.parent
         / "Metadata"
         / f"GridSquare_{grid_square}.dm"
     )
-
-
-def _grid_square_data(xml_path: Path, grid_square: int, session_id: int) -> GridSquare:
-    image_paths = list(
-        (xml_path.parent.parent).glob(
-            f"Images-Disc*/GridSquare_{grid_square}/GridSquare_*.jpg"
-        )
-    )
-    if image_paths:
-        image_paths.sort(key=lambda x: x.stat().st_ctime)
-        image_path = image_paths[-1]
-        with open(Path(image_path).with_suffix(".xml")) as gs_xml:
-            gs_xml_data = xmltodict.parse(gs_xml.read())
-        readout_area = gs_xml_data["MicroscopeImage"]["microscopeData"]["acquisition"][
-            "camera"
-        ]["ReadoutArea"]
-        pixel_size = gs_xml_data["MicroscopeImage"]["SpatialScale"]["pixelSize"]["x"][
-            "numericValue"
-        ]
-        full_size = (int(readout_area["a:width"]), int(readout_area["a:height"]))
-        return GridSquare(
-            id=grid_square,
-            session_id=session_id,
-            readout_area_x=full_size[0] if image_path else None,
-            readout_area_y=full_size[1] if image_path else None,
-            thumbnail_size_x=int((512 / max(full_size)) * full_size[0]),
-            thumbnail_size_y=int((512 / max(full_size)) * full_size[1]),
-            pixel_size=float(pixel_size) if image_path else None,
-            image=str(image_path),
-        )
-    return GridSquare(id=grid_square, session_id=session_id)
-
-
-def _foil_hole_data(
-    xml_path: Path, foil_hole: int, grid_square: int, session_id: int
-) -> FoilHole:
-    with open(xml_path, "r") as xml:
-        for_parsing = xml.read()
-        data = xmltodict.parse(for_parsing)
-    data = data["GridSquareXml"]
-    serialization_array = data["TargetLocations"]["TargetLocationsEfficient"][
-        "a:m_serializationArray"
-    ]
-    required_key = ""
-    for key in serialization_array.keys():
-        if key.startswith("b:KeyValuePairOfintTargetLocation"):
-            required_key = key
-            break
-    if required_key:
-        image_paths = list(
-            (xml_path.parent.parent).glob(
-                f"Images-Disc*/GridSquare_{grid_square}/FoilHoles/FoilHole_{foil_hole}_*.jpg"
-            )
-        )
-        image_paths.sort(key=lambda x: x.stat().st_ctime)
-        image_path: Path | str = image_paths[-1] if image_paths else ""
-        if image_path:
-            with open(Path(image_path).with_suffix(".xml")) as fh_xml:
-                fh_xml_data = xmltodict.parse(fh_xml.read())
-            readout_area = fh_xml_data["MicroscopeImage"]["microscopeData"][
-                "acquisition"
-            ]["camera"]["ReadoutArea"]
-            pixel_size = fh_xml_data["MicroscopeImage"]["SpatialScale"]["pixelSize"][
-                "x"
-            ]["numericValue"]
-            full_size = (int(readout_area["a:width"]), int(readout_area["a:height"]))
-        for fh_block in serialization_array[required_key]:
-            pix = fh_block["b:value"]["PixelCenter"]
-            stage = fh_block["b:value"]["StagePosition"]
-            diameter = fh_block["b:value"]["PixelWidthHeight"]["c:width"]
-            if int(fh_block["b:key"]) == foil_hole:
-                return FoilHole(
-                    id=foil_hole,
-                    grid_square_id=grid_square,
-                    session_id=session_id,
-                    x_location=float(pix["c:x"]),
-                    y_location=float(pix["c:y"]),
-                    x_stage_position=float(stage["c:X"]),
-                    y_stage_position=float(stage["c:Y"]),
-                    readout_area_x=full_size[0] if image_path else None,
-                    readout_area_y=full_size[1] if image_path else None,
-                    thumbnail_size_x=None,
-                    thumbnail_size_y=None,
-                    pixel_size=float(pixel_size) if image_path else None,
-                    image=str(image_path),
-                    diameter=diameter,
-                )
-    logger.warning(
-        f"Foil hole positions could not be determined from metadata file {xml_path} for foil hole {foil_hole}"
-    )
-    return FoilHole(id=foil_hole, grid_square_id=grid_square, session_id=session_id)
+    if not metadata_file.is_file():
+        logger.warning(f"Grid square metadata file {str(metadata_file)} does not exist")
+    return metadata_file
 
 
 def _get_source(file_path: Path, environment: MurfeyInstanceEnvironment) -> Path | None:
+    possible_sources = []
     for s in environment.sources:
         if file_path.is_relative_to(s):
-            return s
-    return None
+            possible_sources.append(s)
+    if not possible_sources:
+        return None
+    elif len(possible_sources) == 1:
+        return possible_sources[0]
+    source = possible_sources[0]
+    for extra_source in possible_sources[1:]:
+        if extra_source.is_relative_to(source):
+            source = extra_source
+    return source
 
 
 def _get_xml_list_index(key: str, xml_list: list) -> int:
@@ -285,34 +92,16 @@ def _get_xml_list_index(key: str, xml_list: list) -> int:
     raise ValueError(f"Key not found in XML list: {key}")
 
 
-class _SPAContext(Context):
+class SPAModularContext(Context):
     user_params = [
         ProcessingParameter(
             "dose_per_frame",
             "Dose Per Frame [e- / Angstrom^2 / frame] (after EER grouping if relevant)",
             default=1,
         ),
-        ProcessingParameter(
-            "estimate_particle_diameter",
-            "Use crYOLO to Estimate Particle Diameter",
-            default=True,
-        ),
-        ProcessingParameter(
-            "particle_diameter", "Particle Diameter (Angstroms)", default=None
-        ),
-        ProcessingParameter("use_cryolo", "Use crYOLO Autopicking", default=True),
         ProcessingParameter("symmetry", "Symmetry Group", default="C1"),
         ProcessingParameter("eer_fractionation", "EER Fractionation", default=20),
-        ProcessingParameter(
-            "mask_diameter", "Mask Diameter (2D classification)", default=190
-        ),
-        ProcessingParameter("boxsize", "Box Size", default=256),
-        ProcessingParameter("downscale", "Downscale Extracted Particles", default=True),
-        ProcessingParameter(
-            "small_boxsize", "Downscaled Extracted Particle Size (pixels)", default=128
-        ),
         ProcessingParameter("gain_ref", "Gain Reference"),
-        ProcessingParameter("gain_ref_superres", "Unbinned Gain Reference"),
     ]
     metadata_params = [
         ProcessingParameter("voltage", "Voltage"),
@@ -326,7 +115,6 @@ class _SPAContext(Context):
         super().__init__("SPA", acquisition_software)
         self._basepath = basepath
         self._processing_job_stash: dict = {}
-        self._preprocessing_triggers: dict = {}
         self._foil_holes: Dict[int, List[int]] = {}
 
     def gather_metadata(
@@ -348,7 +136,6 @@ class _SPAContext(Context):
                 return OrderedDict({})
             data = xmltodict.parse(for_parsing)
         magnification = 0
-        num_fractions = 1
         metadata: OrderedDict = OrderedDict({})
         metadata["experiment_type"] = "SPA"
         if data.get("Acquisition"):
@@ -401,14 +188,6 @@ class _SPAContext(Context):
                 )  # convert e / m^2 to e / A^2
             except ValueError:
                 metadata["total_exposed_dose"] = 1
-            try:
-                num_fractions = int(
-                    data["MicroscopeImage"]["microscopeData"]["acquisition"]["camera"][
-                        "CameraSpecificInput"
-                    ]["a:KeyValueOfstringanyType"][2]["a:Value"]["b:NumberOffractions"]
-                )
-            except (KeyError, IndexError):
-                pass
             c2_index = 3
             for i, el in enumerate(
                 data["MicroscopeImage"]["CustomData"]["a:KeyValueOfstringanyType"]
@@ -447,7 +226,10 @@ class _SPAContext(Context):
         binning_factor = 1
         if environment:
             server_config_response = capture_get(
-                f"{str(environment.url.geturl())}/instruments/{environment.instrument_name}/machine"
+                base_url=str(environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="machine_info_by_instrument",
+                instrument_name=environment.instrument_name,
             )
             if server_config_response is None:
                 return None
@@ -477,97 +259,23 @@ class _SPAContext(Context):
         metadata["image_size_x"] = str(int(metadata["image_size_x"]) * binning_factor)
         metadata["image_size_y"] = str(int(metadata["image_size_y"]) * binning_factor)
         metadata["motion_corr_binning"] = 1 if binning_factor_xml == 2 else 2
-        if environment:
-            metadata["gain_ref"] = (
-                environment.data_collection_parameters.get("gain_ref")
-                if environment
-                and environment.data_collection_parameters.get("gain_ref")
-                not in (None, "None")
-                else f"data/{datetime.now().year}/{environment.visit}/processing/gain.mrc"
-            )
-            metadata["gain_ref_superres"] = (
-                environment.data_collection_parameters.get("gain_ref_superres")
-                if environment
-                and environment.data_collection_parameters.get("gain_ref_superres")
-                not in (None, "None")
-                else f"data/{datetime.now().year}/{environment.visit}/processing/gain_superres.mrc"
-            )
-        else:
-            metadata["gain_ref"] = None
-            metadata["gain_ref_superres"] = None
-        if metadata.get("total_exposed_dose"):
-            metadata["dose_per_frame"] = (
-                environment.data_collection_parameters.get("dose_per_frame")
-                if environment
-                and environment.data_collection_parameters.get("dose_per_frame")
-                not in (None, "None")
-                else round(metadata["total_exposed_dose"] / num_fractions, 3)
-            )
-        else:
-            metadata["dose_per_frame"] = (
-                environment.data_collection_parameters.get("dose_per_frame")
-                if environment
-                else None
-            )
-
-        metadata["use_cryolo"] = (
-            environment.data_collection_parameters.get("use_cryolo")
-            if environment
-            else None
-        ) or True
-        metadata["symmetry"] = (
-            environment.data_collection_parameters.get("symmetry")
-            if environment
-            else None
-        ) or "C1"
-        metadata["mask_diameter"] = (
-            environment.data_collection_parameters.get("mask_diameter")
-            if environment
-            else None
-        ) or 190
-        metadata["boxsize"] = (
-            environment.data_collection_parameters.get("boxsize")
-            if environment
-            else None
-        ) or 256
-        metadata["downscale"] = (
-            environment.data_collection_parameters.get("downscale")
-            if environment
-            else None
-        ) or True
-        metadata["small_boxsize"] = (
-            environment.data_collection_parameters.get("small_boxsize")
-            if environment
-            else None
-        ) or 128
+        metadata["gain_ref"] = environment.gain_ref if environment else None
+        metadata["dose_per_frame"] = environment.dose_per_frame if environment else None
+        metadata["symmetry"] = (environment.symmetry if environment else None) or "C1"
         metadata["eer_fractionation"] = (
-            environment.data_collection_parameters.get("eer_fractionation")
-            if environment
-            else None
+            environment.eer_fractionation if environment else None
         ) or 20
         metadata["source"] = str(self._basepath)
-        metadata["particle_diameter"] = (
-            environment.data_collection_parameters.get("particle_diameter")
-            if environment
-            else None
-        ) or 0
-        metadata["estimate_particle_diameter"] = (
-            environment.data_collection_parameters.get("estimate_particle_diameter")
-            if environment
-            else None
-        ) or True
         return metadata
 
-
-class SPAModularContext(_SPAContext):
     def _position_analysis(
         self,
         transferred_file: Path,
         environment: MurfeyInstanceEnvironment,
         source: Path,
         machine_config: dict,
-    ) -> int:
-        grid_square = _grid_square_from_file(transferred_file)
+    ) -> Optional[int]:
+        grid_square = grid_square_from_file(transferred_file)
         grid_square_metadata_file = _grid_square_metadata_file(
             transferred_file,
             [Path(p) for p in machine_config["data_directories"]],
@@ -590,12 +298,18 @@ class SPAModularContext(_SPAContext):
                 Optional[float],
             ] = (None, None, None, None, None, None, None)
             data_collection_group = (
-                requests.get(
-                    f"{str(environment.url.geturl())}/sessions/{environment.murfey_session}/data_collection_groups"
+                capture_get(
+                    base_url=str(environment.url.geturl()),
+                    router_name="session_info.router",
+                    function_name="get_dc_groups",
+                    session_id=environment.murfey_session,
                 )
                 .json()
                 .get(str(source), {})
             )
+            if not data_collection_group:
+                logger.info("Data collection group has not yet been made")
+                return None
             if data_collection_group.get("atlas"):
                 visit_path = ""
                 for p in transferred_file.parts:
@@ -606,22 +320,23 @@ class SPAModularContext(_SPAContext):
                     local_atlas_path = (
                         Path(visit_path) / environment.samples[source].atlas
                     )
-                    gs_pix_position = _get_grid_square_atlas_positions(
+                    gs_pix_position = get_grid_square_atlas_positions(
                         local_atlas_path,
                         grid_square=str(grid_square),
                     )[str(grid_square)]
-            gs_url = f"{str(environment.url.geturl())}/sessions/{environment.murfey_session}/grid_square/{grid_square}"
-            gs = _grid_square_data(
+            gs = grid_square_data(
                 grid_square_metadata_file,
                 grid_square,
-                environment.murfey_session,
+            )
+            metadata_source_as_str = (
+                "/".join(source.parts[:-2])
+                + f"/{environment.visit}/"
+                + source.parts[-2]
             )
             metadata_source = Path(
-                (
-                    "/".join(source.parts[:-2])
-                    + f"/{environment.visit}/"
-                    + source.parts[-2]
-                )[1:]
+                metadata_source_as_str[1:]
+                if metadata_source_as_str.startswith("//")
+                else metadata_source_as_str
             )
             image_path = (
                 _file_transferred_to(environment, metadata_source, Path(gs.image))
@@ -629,8 +344,12 @@ class SPAModularContext(_SPAContext):
                 else ""
             )
             capture_post(
-                gs_url,
-                json={
+                base_url=str(environment.url.geturl()),
+                router_name="session_control.spa_router",
+                function_name="register_grid_square",
+                session_id=environment.murfey_session,
+                gsid=grid_square,
+                data={
                     "tag": str(source),
                     "readout_area_x": gs.readout_area_x,
                     "readout_area_y": gs.readout_area_y,
@@ -647,25 +366,23 @@ class SPAModularContext(_SPAContext):
                     "angle": gs_pix_position[6],
                 },
             )
-        foil_hole = _foil_hole_from_file(transferred_file)
+        foil_hole = foil_hole_from_file(transferred_file)
         if foil_hole not in self._foil_holes[grid_square]:
-            fh_url = f"{str(environment.url.geturl())}/sessions/{environment.murfey_session}/grid_square/{grid_square}/foil_hole"
-            if (
-                grid_square_metadata_file.is_file()
-                and environment.murfey_session is not None
-            ):
-                fh = _foil_hole_data(
+            if environment.murfey_session is not None:
+                fh = foil_hole_data(
                     grid_square_metadata_file,
                     foil_hole,
                     grid_square,
-                    environment.murfey_session,
+                )
+                metadata_source_as_str = (
+                    "/".join(source.parts[:-2])
+                    + f"/{environment.visit}/"
+                    + source.parts[-2]
                 )
                 metadata_source = Path(
-                    (
-                        "/".join(source.parts[:-2])
-                        + f"/{environment.visit}/"
-                        + source.parts[-2]
-                    )[1:]
+                    metadata_source_as_str[1:]
+                    if metadata_source_as_str.startswith("//")
+                    else metadata_source_as_str
                 )
                 image_path = (
                     _file_transferred_to(environment, metadata_source, Path(fh.image))
@@ -673,8 +390,12 @@ class SPAModularContext(_SPAContext):
                     else ""
                 )
                 capture_post(
-                    fh_url,
-                    json={
+                    base_url=str(environment.url.geturl()),
+                    router_name="session_control.spa_router",
+                    function_name="register_foil_hole",
+                    session_id=environment.murfey_session,
+                    gs_name=grid_square,
+                    data={
                         "name": foil_hole,
                         "x_location": fh.x_location,
                         "y_location": fh.y_location,
@@ -692,8 +413,12 @@ class SPAModularContext(_SPAContext):
                 )
             else:
                 capture_post(
-                    fh_url,
-                    json={
+                    base_url=str(environment.url.geturl()),
+                    router_name="session_control.spa_router",
+                    function_name="register_foil_hole",
+                    session_id=environment.murfey_session,
+                    gs_name=grid_square,
+                    data={
                         "name": foil_hole,
                         "tag": str(source),
                     },
@@ -749,7 +474,9 @@ class SPAModularContext(_SPAContext):
                         )
                         if not environment.movie_counters.get(str(source)):
                             movie_counts_get = capture_get(
-                                f"{str(environment.url.geturl())}/num_movies",
+                                base_url=str(environment.url.geturl()),
+                                router_name="session_control.router",
+                                function_name="count_number_of_movies",
                             )
                             if movie_counts_get is not None:
                                 environment.movie_counters[str(source)] = count(
@@ -763,13 +490,17 @@ class SPAModularContext(_SPAContext):
                         eer_fractionation_file = None
                         if file_transferred_to.suffix == ".eer":
                             response = capture_post(
-                                f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/eer_fractionation_file",
-                                json={
+                                base_url=str(environment.url.geturl()),
+                                router_name="file_io_instrument.router",
+                                function_name="write_eer_fractionation_file",
+                                visit_name=environment.visit,
+                                session_id=environment.murfey_session,
+                                data={
                                     "eer_path": str(file_transferred_to),
-                                    "fractionation": environment.data_collection_parameters[
+                                    "fractionation": self.data_collection_parameters[
                                         "eer_fractionation"
                                     ],
-                                    "dose_per_frame": environment.data_collection_parameters[
+                                    "dose_per_frame": self.data_collection_parameters[
                                         "dose_per_frame"
                                     ],
                                     "fractionation_file_name": "eer_fractionation_spa.txt",
@@ -785,11 +516,14 @@ class SPAModularContext(_SPAContext):
                             foil_hole: Optional[int] = self._position_analysis(
                                 transferred_file, environment, source, machine_config
                             )
-                        except Exception:
+                        except Exception as e:
                             # try to continue if position information gathering fails so that movie is processed anyway
+                            logger.warning(
+                                f"Unable to register foil hole for {str(file_transferred_to)}. Exception: {str(e)}",
+                                exc_info=True,
+                            )
                             foil_hole = None
 
-                        preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/spa_preprocess"
                         preproc_data = {
                             "path": str(file_transferred_to),
                             "description": "",
@@ -798,20 +532,16 @@ class SPAModularContext(_SPAContext):
                             "image_number": environment.movies[
                                 file_transferred_to
                             ].movie_number,
-                            "pixel_size": environment.data_collection_parameters.get(
+                            "pixel_size": self.data_collection_parameters.get(
                                 "pixel_size_on_image"
                             ),
                             "autoproc_program_id": None,
-                            "dose_per_frame": environment.data_collection_parameters.get(
-                                "dose_per_frame"
-                            ),
-                            "mc_binning": environment.data_collection_parameters.get(
+                            "dose_per_frame": environment.dose_per_frame,
+                            "mc_binning": self.data_collection_parameters.get(
                                 "motion_corr_binning", 1
                             ),
-                            "gain_ref": environment.data_collection_parameters.get(
-                                "gain_ref"
-                            ),
-                            "extract_downscale": environment.data_collection_parameters.get(
+                            "gain_ref": environment.gain_ref,
+                            "extract_downscale": self.data_collection_parameters.get(
                                 "downscale", True
                             ),
                             "eer_fractionation_file": eer_fractionation_file,
@@ -819,8 +549,12 @@ class SPAModularContext(_SPAContext):
                             "foil_hole_id": foil_hole,
                         }
                         capture_post(
-                            preproc_url,
-                            json={
+                            base_url=str(environment.url.geturl()),
+                            router_name="workflow.spa_router",
+                            function_name="request_spa_preprocessing",
+                            visit_name=environment.visit,
+                            session_id=environment.murfey_session,
+                            data={
                                 k: None if v == "None" else v
                                 for k, v in preproc_data.items()
                             },
@@ -853,104 +587,3 @@ class SPAModularContext(_SPAContext):
         url: str = "",
     ):
         return
-
-
-class SPAContext(_SPAContext):
-    def _register_data_collection(
-        self,
-        tag: str,
-        url: str,
-        data: dict,
-        environment: MurfeyInstanceEnvironment,
-    ):
-        logger.info(f"registering data collection with data {data}")
-        environment.id_tag_registry["data_collection"].append(tag)
-        image_directory = str(environment.default_destinations[Path(tag)])
-        json = {
-            "voltage": data["voltage"],
-            "pixel_size_on_image": data["pixel_size_on_image"],
-            "experiment_type": data["experiment_type"],
-            "image_size_x": data["image_size_x"],
-            "image_size_y": data["image_size_y"],
-            "file_extension": data["file_extension"],
-            "acquisition_software": data["acquisition_software"],
-            "image_directory": image_directory,
-            "tag": tag,
-            "source": tag,
-            "magnification": data["magnification"],
-            "total_exposed_dose": data.get("total_exposed_dose"),
-            "c2aperture": data.get("c2aperture"),
-            "exposure_time": data.get("exposure_time"),
-            "slit_width": data.get("slit_width"),
-            "phase_plate": data.get("phase_plate", False),
-        }
-        capture_post(url, json=json)
-
-    def post_transfer(
-        self,
-        transferred_file: Path,
-        environment: MurfeyInstanceEnvironment | None = None,
-        **kwargs,
-    ) -> bool:
-        return True
-
-    def _register_processing_job(
-        self,
-        tag: str,
-        environment: MurfeyInstanceEnvironment,
-        parameters: Dict[str, Any] | None = None,
-    ):
-        logger.info(f"registering processing job with parameters: {parameters}")
-        parameters = parameters or {}
-        environment.id_tag_registry["processing_job"].append(tag)
-        proc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/register_processing_job"
-        machine_config = get_machine_config_client(
-            str(environment.url.geturl()),
-            instrument_name=environment.instrument_name,
-            demo=environment.demo,
-        )
-        image_directory = str(
-            Path(machine_config.get("rsync_basepath", "."))
-            / environment.default_destinations[Path(tag)]
-        )
-        if self._acquisition_software == "epu":
-            import_images = f"{Path(image_directory).resolve()}/GridSquare*/Data/*{parameters['file_extension']}"
-        else:
-            import_images = (
-                f"{Path(image_directory).resolve()}/*{parameters['file_extension']}"
-            )
-        msg: Dict[str, Any] = {
-            "tag": tag,
-            "source": tag,
-            "recipe": "ispyb-relion",
-            "parameters": {
-                "acquisition_software": parameters["acquisition_software"],
-                "voltage": parameters["voltage"],
-                "gain_ref": parameters["gain_ref"],
-                "dose_per_frame": parameters["dose_per_frame"],
-                "eer_grouping": parameters["eer_fractionation"],
-                "import_images": import_images,
-                "angpix": float(parameters["pixel_size_on_image"]) * 1e10,
-                "symmetry": parameters["symmetry"],
-                "boxsize": parameters["boxsize"],
-                "downscale": parameters["downscale"],
-                "small_boxsize": parameters["small_boxsize"],
-                "mask_diameter": parameters["mask_diameter"],
-                "use_cryolo": parameters["use_cryolo"],
-                "estimate_particle_diameter": parameters["estimate_particle_diameter"],
-            },
-        }
-        if parameters["particle_diameter"]:
-            msg["parameters"]["particle_diameter"] = parameters["particle_diameter"]
-        capture_post(proc_url, json=msg)
-
-    def _launch_spa_pipeline(
-        self,
-        tag: str,
-        jobid: int,
-        environment: MurfeyInstanceEnvironment,
-        url: str = "",
-    ):
-        environment.id_tag_registry["auto_proc_program"].append(tag)
-        data = {"job_id": jobid}
-        capture_post(url, json=data)
