@@ -1,14 +1,11 @@
 from __future__ import annotations
 
 import logging
-from datetime import datetime
 from pathlib import Path
 from threading import RLock
 from typing import Callable, Dict, List, OrderedDict
 
-import requests
 import xmltodict
-from pydantic import BaseModel
 
 import murfey.util.eer
 from murfey.client.context import Context, ProcessingParameter
@@ -17,15 +14,12 @@ from murfey.client.instance_environment import (
     MovieTracker,
     MurfeyID,
     MurfeyInstanceEnvironment,
-    global_env_lock,
 )
-from murfey.util import authorised_requests, capture_post, get_machine_config_client
+from murfey.util.client import capture_get, capture_post, get_machine_config_client
 from murfey.util.mdoc import get_block, get_global_data, get_num_blocks
 
 logger = logging.getLogger("murfey.client.contexts.tomo")
 
-requests.get, requests.post, requests.put, requests.delete = authorised_requests()
-
 
 def _get_tilt_angle_v5_7(p: Path) -> str:
     return p.name.split("[")[1].split("]")[0]
@@ -64,35 +58,11 @@ def _construct_tilt_series_name(file_path: Path) -> str:
     return "_".join(split_name[:-5])
 
 
-def _midpoint(angles: List[float]) -> int:
-    if not angles:
-        return 0
-    if len(angles) <= 2:
-        return round(angles[0])
-    sorted_angles = sorted(angles)
-    return round(
-        sorted_angles[len(sorted_angles) // 2]
-        if sorted_angles[len(sorted_angles) // 2]
-        and sorted_angles[len(sorted_angles) // 2 + 1]
-        else 0
-    )
-
-
-class ProcessFileIncomplete(BaseModel):
-    dest: Path
-    source: Path
-    image_number: int
-    mc_uuid: int
-    tag: str
-    description: str = ""
-
-
 class TomographyContext(Context):
     user_params = [
         ProcessingParameter(
             "dose_per_frame", "Dose Per Frame (e- / Angstrom^2 / frame)", default=1
         ),
-        ProcessingParameter("manual_tilt_offset", "Tilt Offset", default=0),
         ProcessingParameter("gain_ref", "Gain Reference"),
         ProcessingParameter("eer_fractionation", "EER Fractionation", default=20),
     ]
@@ -102,6 +72,8 @@ class TomographyContext(Context):
         ProcessingParameter("image_size_y", "Image Size Y"),
         ProcessingParameter("pixel_size_on_image", "Pixel Size"),
         ProcessingParameter("motion_corr_binning", "Motion Correction Binning"),
+        ProcessingParameter("frame_count", "Number of image frames"),
+        ProcessingParameter("tilt_axis", "Stage rotation angle"),
         ProcessingParameter("num_eer_frames", "Number of EER Frames"),
     ]
 
@@ -109,113 +81,106 @@ class TomographyContext(Context):
         super().__init__("Tomography", acquisition_software)
         self._basepath = basepath
         self._tilt_series: Dict[str, List[Path]] = {}
+        self._tilt_series_with_pjids: List[str] = []
         self._tilt_series_sizes: Dict[str, int] = {}
         self._completed_tilt_series: List[str] = []
         self._aligned_tilt_series: List[str] = []
         self._data_collection_stash: list = []
         self._processing_job_stash: dict = {}
-        self._preprocessing_triggers: dict = {}
         self._lock: RLock = RLock()
 
-    def _flush_data_collections(self):
-        logger.info(
-            f"Flushing {len(self._data_collection_stash)} data collection API calls"
-        )
-        for dc_data in self._data_collection_stash:
-            data = {
-                **dc_data[2],
-                **{
-                    k: v
-                    for k, v in dc_data[1].data_collection_parameters.items()
-                    if k != "tag"
-                },
-            }
-            capture_post(dc_data[0], json=data)
-        self._data_collection_stash = []
-
-    def _flush_processing_job(self, tag: str):
-        if proc_data := self._processing_job_stash.get(tag):
-            for pd in proc_data:
-                requests.post(pd[0], json=pd[1])
-            self._processing_job_stash.pop(tag)
-
-    def _flush_processing_jobs(self):
-        logger.info(
-            f"Flushing {len(self._processing_job_stash.keys())} processing job API calls"
-        )
-        for v in self._processing_job_stash.values():
-            for pd in v:
-                requests.post(pd[0], json=pd[1])
-        self._processing_job_stash = {}
-
-    def _flush_preprocess(self, tag: str, app_id: int):
-        if tag_tr := self._preprocessing_triggers.get(tag):
-            for tr in tag_tr:
-                process_file = self._complete_process_file(tr[1], tr[2], app_id)
-                if process_file:
-                    capture_post(tr[0], json=process_file)
-            self._preprocessing_triggers.pop(tag)
-
-    def _complete_process_file(
+    def register_tomography_data_collections(
         self,
-        incomplete_process_file: ProcessFileIncomplete,
-        environment: MurfeyInstanceEnvironment,
-        app_id: int,
-    ) -> dict:
+        file_extension: str,
+        image_directory: str,
+        environment: MurfeyInstanceEnvironment | None = None,
+    ):
+        if not environment:
+            logger.error(
+                "No environment passed to register tomography data collections"
+            )
+            return
         try:
-            with global_env_lock:
-                tag = incomplete_process_file.tag
+            dcg_data = {
+                "experiment_type": "tomo",
+                "experiment_type_id": 36,
+                "tag": str(self._basepath),
+                "atlas": "",
+                "sample": None,
+            }
+            capture_post(
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.router",
+                function_name="register_dc_group",
+                visit_name=environment.visit,
+                session_id=environment.murfey_session,
+                data=dcg_data,
+            )
 
-                eer_fractionation_file = None
-                if environment.data_collection_parameters.get("num_eer_frames"):
-                    response = requests.post(
-                        f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/eer_fractionation_file",
-                        json={
-                            "num_frames": environment.data_collection_parameters[
-                                "num_eer_frames"
-                            ],
-                            "fractionation": environment.data_collection_parameters[
-                                "eer_fractionation"
-                            ],
-                            "dose_per_frame": environment.data_collection_parameters[
-                                "dose_per_frame"
-                            ],
-                            "fractionation_file_name": "eer_fractionation_tomo.txt",
-                        },
-                    )
-                    eer_fractionation_file = response.json()["eer_fractionation_file"]
+            for tilt_series in self._tilt_series.keys():
+                if tilt_series not in self._tilt_series_with_pjids:
+                    dc_data = {
+                        "experiment_type": "tomography",
+                        "file_extension": file_extension,
+                        "acquisition_software": self._acquisition_software,
+                        "image_directory": image_directory,
+                        "data_collection_tag": tilt_series,
+                        "source": str(self._basepath),
+                        "tag": tilt_series,
+                    }
+                    if (
+                        self.data_collection_parameters
+                        and self.data_collection_parameters.get("voltage")
+                    ):
+                        # Once mdoc parameters are known register processing jobs
+                        dc_data.update(
+                            {
+                                "voltage": self.data_collection_parameters["voltage"],
+                                "pixel_size_on_image": self.data_collection_parameters[
+                                    "pixel_size_on_image"
+                                ],
+                                "image_size_x": self.data_collection_parameters[
+                                    "image_size_x"
+                                ],
+                                "image_size_y": self.data_collection_parameters[
+                                    "image_size_y"
+                                ],
+                                "magnification": self.data_collection_parameters[
+                                    "magnification"
+                                ],
+                            }
+                        )
+                        capture_post(
+                            base_url=str(environment.url.geturl()),
+                            router_name="workflow.router",
+                            function_name="start_dc",
+                            visit_name=environment.visit,
+                            session_id=environment.murfey_session,
+                            data=dc_data,
+                        )
 
-                new_dict = {
-                    "path": str(incomplete_process_file.dest),
-                    "description": incomplete_process_file.description,
-                    "size": incomplete_process_file.source.stat().st_size,
-                    "timestamp": incomplete_process_file.source.stat().st_ctime,
-                    "processing_job": environment.processing_job_ids[tag][
-                        "em-tomo-preprocess"
-                    ],
-                    "data_collection_id": environment.data_collection_ids[tag],
-                    "image_number": incomplete_process_file.image_number,
-                    "pixel_size": environment.data_collection_parameters[
-                        "pixel_size_on_image"
-                    ],
-                    "autoproc_program_id": app_id,
-                    "mc_uuid": incomplete_process_file.mc_uuid,
-                    "dose_per_frame": environment.data_collection_parameters.get(
-                        "dose_per_frame"
-                    ),
-                    "mc_binning": environment.data_collection_parameters.get(
-                        "motion_corr_binning", 1
-                    ),
-                    "gain_ref": environment.data_collection_parameters.get("gain_ref"),
-                    "voltage": environment.data_collection_parameters.get(
-                        "voltage", 300
-                    ),
-                    "eer_fractionation_file": eer_fractionation_file,
-                }
-                return new_dict
-        except KeyError:
-            logger.warning("Key error encountered in _complete_process_file")
-            return {}
+                        for recipe in ("em-tomo-preprocess", "em-tomo-align"):
+                            capture_post(
+                                base_url=str(environment.url.geturl()),
+                                router_name="workflow.router",
+                                function_name="register_proc",
+                                visit_name=environment.visit,
+                                session_id=environment.murfey_session,
+                                data={
+                                    "tag": tilt_series,
+                                    "source": str(self._basepath),
+                                    "recipe": recipe,
+                                    "experiment_type": "tomography",
+                                },
+                            )
+                        self._tilt_series_with_pjids.append(tilt_series)
+                    else:
+                        logger.info(
+                            "Cannot register data collection yet as no values from mdoc"
+                        )
+
+        except Exception as e:
+            logger.error(f"ERROR {e}, {self.data_collection_parameters}", exc_info=True)
 
     def _file_transferred_to(
         self, environment: MurfeyInstanceEnvironment, source: Path, file_path: Path
@@ -302,13 +267,18 @@ class TomographyContext(Context):
                 f"Tilt series {tilt_series} was previously thought complete but now {file_path} has been seen"
             )
             self._completed_tilt_series.remove(tilt_series)
-            rerun_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/rerun_tilt_series"
             rerun_data = {
                 "session_id": environment.murfey_session,
                 "tag": tilt_series,
                 "source": str(file_path.parent),
             }
-            capture_post(rerun_url, json=rerun_data)
+            capture_post(
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_tilt_series_for_rerun",
+                visit_name=environment.visit,
+                data=rerun_data,
+            )
             if tilt_series in self._aligned_tilt_series:
                 with self._lock:
                     self._aligned_tilt_series.remove(tilt_series)
@@ -316,105 +286,31 @@ class TomographyContext(Context):
         if not self._tilt_series.get(tilt_series):
             logger.info(f"New tilt series found: {tilt_series}")
             self._tilt_series[tilt_series] = [file_path]
-            ts_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/tilt_series"
             ts_data = {
                 "session_id": environment.murfey_session,
                 "tag": tilt_series,
                 "source": str(file_path.parent),
             }
-            capture_post(ts_url, json=ts_data)
+            capture_post(
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_tilt_series",
+                visit_name=environment.visit,
+                data=ts_data,
+            )
             if not self._tilt_series_sizes.get(tilt_series):
                 self._tilt_series_sizes[tilt_series] = 0
-            try:
-                if environment:
-                    url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/start_data_collection"
-                    data = {
-                        "experiment_type": "tomography",
-                        "file_extension": file_path.suffix,
-                        "acquisition_software": self._acquisition_software,
-                        "image_directory": str(
-                            environment.default_destinations.get(
-                                file_path.parent, file_path.parent
-                            )
-                        ),
-                        "data_collection_tag": tilt_series,
-                        "source": str(self._basepath),
-                        "tag": tilt_series,
-                    }
-                    if (
-                        environment.data_collection_parameters
-                        and environment.data_collection_parameters.get("voltage")
-                    ):
-                        data.update(
-                            {
-                                "voltage": environment.data_collection_parameters[
-                                    "voltage"
-                                ],
-                                "pixel_size_on_image": environment.data_collection_parameters[
-                                    "pixel_size_on_image"
-                                ],
-                                "image_size_x": environment.data_collection_parameters[
-                                    "image_size_x"
-                                ],
-                                "image_size_y": environment.data_collection_parameters[
-                                    "image_size_y"
-                                ],
-                                "magnification": environment.data_collection_parameters[
-                                    "magnification"
-                                ],
-                            }
-                        )
-                    proc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/register_processing_job"
-                    if (
-                        environment.data_collection_group_ids.get(str(self._basepath))
-                        is None
-                    ):
-                        self._data_collection_stash.append((url, environment, data))
-                        self._processing_job_stash[tilt_series] = [
-                            (
-                                proc_url,
-                                {
-                                    "tag": tilt_series,
-                                    "source": str(self._basepath),
-                                    "recipe": "em-tomo-preprocess",
-                                    "experiment_type": "tomography",
-                                },
-                            )
-                        ]
-                        self._processing_job_stash[tilt_series].append(
-                            (
-                                proc_url,
-                                {
-                                    "tag": tilt_series,
-                                    "source": str(self._basepath),
-                                    "recipe": "em-tomo-align",
-                                    "experiment_type": "tomography",
-                                },
-                            )
-                        )
-                    else:
-                        capture_post(url, json=data)
-                        capture_post(
-                            proc_url,
-                            json={
-                                "tag": tilt_series,
-                                "source": str(self._basepath),
-                                "recipe": "em-tomo-preprocess",
-                                "experiment_type": "tomography",
-                            },
-                        )
-                        capture_post(
-                            proc_url,
-                            json={
-                                "tag": tilt_series,
-                                "source": str(self._basepath),
-                                "recipe": "em-tomo-align",
-                                "experiment_type": "tomography",
-                            },
-                        )
 
-            except Exception as e:
-                logger.error(f"ERROR {e}, {environment.data_collection_parameters}")
+            # Will register processing jobs for all tilt series except the first one
+            self.register_tomography_data_collections(
+                file_extension=file_path.suffix,
+                image_directory=str(
+                    environment.default_destinations.get(
+                        file_path.parent, file_path.parent
+                    )
+                ),
+                environment=environment,
+            )
         else:
             if file_path not in self._tilt_series[tilt_series]:
                 for p in self._tilt_series[tilt_series]:
@@ -424,57 +320,65 @@ class TomographyContext(Context):
                     self._tilt_series[tilt_series].append(file_path)
 
         if environment:
-            tilt_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/tilt"
             tilt_data = {
                 "movie_path": str(file_transferred_to),
                 "tilt_series_tag": tilt_series,
                 "source": str(file_path.parent),
             }
-            capture_post(tilt_url, json=tilt_data)
+            capture_post(
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_tilt",
+                visit_name=environment.visit,
+                session_id=environment.murfey_session,
+                data=tilt_data,
+            )
 
             eer_fractionation_file = None
-            if environment.data_collection_parameters.get("num_eer_frames"):
-                response = requests.post(
-                    f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/eer_fractionation_file",
-                    json={
-                        "num_frames": environment.data_collection_parameters[
-                            "num_eer_frames"
-                        ],
-                        "fractionation": environment.data_collection_parameters[
+            if self.data_collection_parameters.get("num_eer_frames"):
+                response = capture_post(
+                    base_url=str(environment.url.geturl()),
+                    router_name="file_io_instrument.router",
+                    function_name="write_eer_fractionation_file",
+                    visit_name=environment.visit,
+                    session_id=environment.murfey_session,
+                    data={
+                        "num_frames": self.data_collection_parameters["num_eer_frames"],
+                        "fractionation": self.data_collection_parameters[
                             "eer_fractionation"
                         ],
-                        "dose_per_frame": environment.data_collection_parameters[
-                            "dose_per_frame"
-                        ],
+                        "dose_per_frame": environment.dose_per_frame or 0,
                         "fractionation_file_name": "eer_fractionation_tomo.txt",
                     },
                 )
                 eer_fractionation_file = response.json()["eer_fractionation_file"]
-            preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/tomography_preprocess"
             preproc_data = {
                 "path": str(file_transferred_to),
                 "description": "",
-                "data_collection_id": environment.data_collection_ids.get(tilt_series),
                 "image_number": environment.movies[file_transferred_to].movie_number,
-                "pixel_size": environment.data_collection_parameters.get(
+                "pixel_size": self.data_collection_parameters.get(
                     "pixel_size_on_image", 0
                 ),
-                "autoproc_program_id": environment.autoproc_program_ids.get(
-                    tilt_series, {}
-                ).get("em-tomo-preprocess"),
-                "dose_per_frame": environment.data_collection_parameters.get(
-                    "dose_per_frame", 0
-                ),
-                "mc_binning": environment.data_collection_parameters.get(
+                "dose_per_frame": environment.dose_per_frame or 0,
+                "frame_count": self.data_collection_parameters.get("frame_count", 0),
+                "tilt_axis": self.data_collection_parameters.get("tilt_axis", 85),
+                "mc_binning": self.data_collection_parameters.get(
                     "motion_corr_binning", 1
                 ),
-                "gain_ref": environment.data_collection_parameters.get("gain_ref"),
-                "voltage": environment.data_collection_parameters.get("voltage", 300),
+                "gain_ref": environment.gain_ref,
+                "voltage": self.data_collection_parameters.get("voltage", 300),
                 "eer_fractionation_file": eer_fractionation_file,
                 "tag": tilt_series,
                 "group_tag": str(self._basepath),
             }
-            capture_post(preproc_url, json=preproc_data)
+            capture_post(
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="request_tomography_preprocessing",
+                visit_name=environment.visit,
+                session_id=environment.murfey_session,
+                data=preproc_data,
+            )
 
         return self._check_tilt_series(tilt_series)
 
@@ -485,6 +389,7 @@ class TomographyContext(Context):
         newly_completed_series: List[str] = []
         mdoc_tilt_series_size = self._tilt_series_sizes.get(tilt_series, 0)
         if not self._tilt_series or not mdoc_tilt_series_size:
+            logger.debug(f"Tilt series size not yet set for {tilt_series!r}")
             return newly_completed_series
 
         counted_tilts = len(self._tilt_series.get(tilt_series, []))
@@ -492,6 +397,11 @@ class TomographyContext(Context):
         if tilt_series_size_check and tilt_series not in self._completed_tilt_series:
             self._completed_tilt_series.append(tilt_series)
             newly_completed_series.append(tilt_series)
+        else:
+            logger.debug(
+                f"{tilt_series!r} not complete yet. Counted {counted_tilts} tilts. "
+                f"Expected number of tilts was {mdoc_tilt_series_size}"
+            )
         return newly_completed_series
 
     def _add_tomo_tilt(
@@ -523,33 +433,6 @@ class TomographyContext(Context):
             required_strings=required_strings,
         )
 
-    def _add_serialem_tilt(
-        self, file_path: Path, environment: MurfeyInstanceEnvironment | None = None
-    ) -> List[str]:
-        delimiters = ("_", "-")
-        for d in delimiters:
-            if file_path.name.count(d) > 1:
-                delimiter = d
-                break
-        else:
-            delimiter = delimiters[0]
-
-        def _extract_tilt_series(p: Path) -> str:
-            split = p.name.split(delimiter)
-            for s in split:
-                if s.isdigit():
-                    return s
-            raise ValueError(
-                f"No digits found in {p.name} after splitting on {delimiter}"
-            )
-
-        return self._add_tilt(
-            file_path,
-            lambda x: ".".join(x.name.split(delimiter)[-1].split(".")[:-1]),
-            environment=environment,
-            required_strings=[],
-        )
-
     def post_transfer(
         self,
         transferred_file: Path,
@@ -587,10 +470,8 @@ class TomographyContext(Context):
                         required_strings=kwargs.get("required_strings")
                         or required_strings,
                     )
-                elif self._acquisition_software == "serialem":
-                    completed_tilts = self._add_serialem_tilt(
-                        transferred_file, environment=environment
-                    )
+                else:
+                    logger.warning(f"Unknown data file {transferred_file}")
             if transferred_file.suffix == ".mdoc":
                 with open(transferred_file, "r") as md:
                     tilt_series = transferred_file.stem
@@ -602,10 +483,12 @@ class TomographyContext(Context):
 
                     # Always update the tilt series length in the database after an mdoc
                     if environment.murfey_session is not None:
-                        length_url = f"{str(environment.url.geturl())}/sessions/{environment.murfey_session}/tilt_series_length"
                         capture_post(
-                            length_url,
-                            json={
+                            base_url=str(environment.url.geturl()),
+                            router_name="workflow.tomo_router",
+                            function_name="register_tilt_series_length",
+                            session_id=environment.murfey_session,
+                            data={
                                 "tags": [tilt_series],
                                 "source": str(transferred_file.parent),
                                 "tilt_series_lengths": [
@@ -619,10 +502,13 @@ class TomographyContext(Context):
                 f"The following tilt series are considered complete: {completed_tilts} "
                 f"after {transferred_file}"
             )
-            complete_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/completed_tilt_series"
             capture_post(
-                complete_url,
-                json={
+                base_url=str(environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_completed_tilt_series",
+                visit_name=environment.visit,
+                session_id=environment.murfey_session,
+                data={
                     "tags": completed_tilts,
                     "source": str(transferred_file.parent),
                     "tilt_series_lengths": [
@@ -656,7 +542,9 @@ class TomographyContext(Context):
                     try:
                         for_parsing = xml.read()
                     except Exception:
-                        logger.warning(f"Failed to parse file {metadata_file}")
+                        logger.warning(
+                            f"Failed to parse file {metadata_file}", exc_info=True
+                        )
                         return OrderedDict({})
                     data = xmltodict.parse(for_parsing)
                 try:
@@ -675,11 +563,8 @@ class TomographyContext(Context):
                     metadata["motion_corr_binning"] = 1
                     metadata["gain_ref"] = None
                     metadata["dose_per_frame"] = (
-                        environment.data_collection_parameters.get("dose_per_frame")
-                        if environment
-                        else None
+                        environment.dose_per_frame if environment else None
                     )
-                    metadata["manual_tilt_offset"] = 0
                     metadata["source"] = str(self._basepath)
                 except KeyError:
                     return OrderedDict({})
@@ -695,14 +580,19 @@ class TomographyContext(Context):
             mdoc_metadata: OrderedDict = OrderedDict({})
             mdoc_metadata["experiment_type"] = "tomography"
             mdoc_metadata["voltage"] = float(mdoc_data["Voltage"])
+            mdoc_metadata["frame_count"] = int(mdoc_data_block["NumSubFrames"])
+            mdoc_metadata["tilt_axis"] = float(mdoc_data_block["RotationAngle"])
             mdoc_metadata["image_size_x"] = int(mdoc_data["ImageSize"][0])
             mdoc_metadata["image_size_y"] = int(mdoc_data["ImageSize"][1])
             mdoc_metadata["magnification"] = int(mdoc_data_block["Magnification"])
             superres_binning = int(mdoc_data_block["Binning"])
             binning_factor = 1
             if environment:
-                server_config = requests.get(
-                    f"{str(environment.url.geturl())}/instruments/{environment.instrument_name}/machine"
+                server_config = capture_get(
+                    base_url=str(environment.url.geturl()),
+                    router_name="session_control.router",
+                    function_name="machine_info_by_instrument",
+                    instrument_name=environment.instrument_name,
                 ).json()
                 if (
                     server_config.get("superres")
@@ -724,22 +614,9 @@ class TomographyContext(Context):
                     float(mdoc_data["PixelSpacing"]) * 1e-10
                 )
             mdoc_metadata["motion_corr_binning"] = binning_factor
-            if environment:
-                mdoc_metadata["gain_ref"] = (
-                    environment.data_collection_parameters.get("gain_ref")
-                    if environment.data_collection_parameters.get("gain_ref")
-                    not in (None, "None")
-                    else f"data/{datetime.now().year}/{environment.visit}/processing/gain.mrc"
-                )
-            else:
-                mdoc_metadata["gain_ref"] = None
+            mdoc_metadata["gain_ref"] = environment.gain_ref if environment else None
             mdoc_metadata["dose_per_frame"] = (
-                environment.data_collection_parameters.get("dose_per_frame")
-                if environment
-                else None
-            )
-            mdoc_metadata["manual_tilt_offset"] = -_midpoint(
-                [float(b["TiltAngle"]) for b in blocks]
+                environment.dose_per_frame if environment else None
             )
             mdoc_metadata["source"] = str(self._basepath)
             mdoc_metadata["tag"] = str(self._basepath)
@@ -752,9 +629,7 @@ class TomographyContext(Context):
                 f".{mdoc_data_block['SubFramePath'].split('.')[-1]}"
             )
             mdoc_metadata["eer_fractionation"] = (
-                environment.data_collection_parameters.get("eer_fractionation")
-                if environment
-                else None
+                environment.eer_fractionation if environment else None
             ) or 20
 
             data_file = mdoc_data_block["SubFramePath"].split("\\")[-1]
@@ -762,8 +637,14 @@ class TomographyContext(Context):
                 mdoc_metadata["num_eer_frames"] = murfey.util.eer.num_frames(
                     metadata_file.parent / data_file
                 )
+                mdoc_metadata["frame_count"] = int(
+                    int(mdoc_metadata["eer_fractionation"])
+                    / int(mdoc_metadata["num_eer_frames"])
+                )
         except Exception as e:
-            logger.error(f"Exception encountered in metadata gathering: {str(e)}")
+            logger.error(
+                f"Exception encountered in metadata gathering: {str(e)}", exc_info=True
+            )
             return OrderedDict({})
 
         return mdoc_metadata
