@@ -2,6 +2,7 @@ import json
 import logging
 import subprocess
 import threading
+import time
 from dataclasses import dataclass, field
 from datetime import datetime
 from functools import partial
@@ -9,19 +10,23 @@ from pathlib import Path
 from typing import Dict, List, Optional
 from urllib.parse import urlparse
 
-import requests
-
 import murfey.client.websocket
 from murfey.client.analyser import Analyser
-from murfey.client.contexts.spa import SPAContext, SPAModularContext
+from murfey.client.contexts.spa import SPAModularContext
 from murfey.client.contexts.tomo import TomographyContext
 from murfey.client.instance_environment import MurfeyInstanceEnvironment
 from murfey.client.rsync import RSyncer, RSyncerUpdate, TransferResult
 from murfey.client.tui.screens import determine_default_destination
 from murfey.client.watchdir import DirWatcher
-from murfey.util import capture_post, posix_path
+from murfey.util import posix_path
+from murfey.util.client import (
+    capture_delete,
+    capture_get,
+    capture_post,
+    get_machine_config_client,
+)
 
-log = logging.getLogger("murfey.client.mutligrid_control")
+log = logging.getLogger("murfey.client.multigrid_control")
 
 
 @dataclass
@@ -32,7 +37,11 @@ class MultigridController:
     session_id: int
     murfey_url: str = "http://localhost:8000"
     rsync_url: str = ""
+    rsync_module: str = "data"
     demo: bool = False
+    finalising: bool = False
+    dormant: bool = False
+    multigrid_watcher_active: bool = True
     processing_enabled: bool = True
     do_transfer: bool = True
     dummy_dc: bool = False
@@ -43,34 +52,36 @@ class MultigridController:
     data_collection_parameters: dict = field(default_factory=lambda: {})
     token: str = ""
     _machine_config: dict = field(default_factory=lambda: {})
+    visit_end_time: Optional[datetime] = None
 
     def __post_init__(self):
-        if self.token:
-            requests.get = partial(
-                requests.get, headers={"Authorization": f"Bearer {self.token}"}
-            )
-            requests.post = partial(
-                requests.post, headers={"Authorization": f"Bearer {self.token}"}
-            )
-            requests.delete = partial(
-                requests.delete, headers={"Authorization": f"Bearer {self.token}"}
-            )
-        machine_data = requests.get(
-            f"{self.murfey_url}/instruments/{self.instrument_name}/machine"
+        machine_data = capture_get(
+            base_url=self.murfey_url,
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=self.instrument_name,
         ).json()
         self.rsync_url = machine_data.get("rsync_url", "")
+        self.rsync_module = machine_data.get("rsync_module", "data")
         self._environment = MurfeyInstanceEnvironment(
             url=urlparse(self.murfey_url, allow_fragments=False),
             client_id=0,
             murfey_session=self.session_id,
             software_versions=machine_data.get("software_versions", {}),
-            default_destination=f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}",
             demo=self.demo,
             visit=self.visit,
-            data_collection_parameters=self.data_collection_parameters,
+            dose_per_frame=self.data_collection_parameters.get("dose_per_frame"),
+            gain_ref=self.data_collection_parameters.get("gain_ref"),
+            symmetry=self.data_collection_parameters.get("symmetry"),
+            eer_fractionation=self.data_collection_parameters.get("eer_fractionation"),
             instrument_name=self.instrument_name,
             # processing_only_mode=server_routing_prefix_found,
         )
+        self._machine_config = get_machine_config_client(
+            str(self._environment.url.geturl()),
+            instrument_name=self._environment.instrument_name,
+            demo=self._environment.demo,
+        )
         self._data_suffixes = (".mrc", ".tiff", ".tif", ".eer")
         self._data_substrings = [
             s
@@ -88,6 +99,127 @@ class MultigridController:
             register_client=False,
         )
 
+        # Calculate the time offset between the client and the server
+        current_time = datetime.now()
+        server_timestamp = capture_get(
+            base_url=self.murfey_url,
+            router_name="session_control.router",
+            function_name="get_current_timestamp",
+        ).json()["timestamp"]
+        self.server_time_offset = current_time - datetime.fromtimestamp(
+            server_timestamp
+        )
+
+        # Store the visit end time in the current device's equivalent time
+        if self.visit_end_time:
+            self.visit_end_time += self.server_time_offset
+
+    def _multigrid_watcher_finalised(self):
+        self.multigrid_watcher_active = False
+
+    def is_ready_for_dormancy(self):
+        """
+        When the multigrid watcher is no longer active, sends a request to safely stop
+        the analyser and file watcher threads, then checks to see that those threads
+        and the RSyncer processes associated with the current session have all been
+        safely stopped
+        """
+        log.debug(
+            f"Starting dormancy check for MultigridController for session {self.session_id}"
+        )
+        if not self.multigrid_watcher_active:
+            for a in self.analysers.values():
+                if a.is_safe_to_stop():
+                    a.stop()
+            for w in self._environment.watchers.values():
+                if w.is_safe_to_stop():
+                    w.stop()
+            return (
+                all(r._finalised for r in self.rsync_processes.values())
+                and not any(a.thread.is_alive() for a in self.analysers.values())
+                and not any(
+                    w.thread.is_alive() for w in self._environment.watchers.values()
+                )
+            )
+        log.debug(f"Multigrid watcher for session {self.session_id} is still active")
+        return False
+
+    def clean_up_once_dormant(self, running_threads: list[threading.Thread]):
+        """
+        A function run in a separate thread that runs the post-session cleanup logic
+        once all threads associated with this current session are halted, and marks
+        the controller as being fully dormant after doing so.
+        """
+        for thread in running_threads:
+            thread.join()
+            log.debug(f"RSyncer cleanup thread {thread.ident} has stopped safely")
+        while not self.is_ready_for_dormancy():
+            time.sleep(10)
+
+        # Once all threads are stopped, remove session from the database
+        log.debug(
+            f"Submitting request to remove session {self.session_id} from database"
+        )
+        response = capture_delete(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="remove_session",
+            session_id=self.session_id,
+        )
+        success = response.status_code == 200 if response else False
+        if not success:
+            log.warning(f"Could not delete database data for {self.session_id}")
+
+        # Send message to frontend to trigger a refresh
+        self.ws.send(
+            json.dumps(
+                {
+                    "message": "refresh",
+                    "target": "sessions",
+                    "instrument_name": self.instrument_name,
+                }
+            )
+        )
+
+        # Mark as dormant
+        self.dormant = True
+
+    def abandon(self):
+        for a in self.analysers.values():
+            a.request_stop()
+        for w in self._environment.watchers.values():
+            w.request_stop()
+        for p in self.rsync_processes.values():
+            p.request_stop()
+
+    def finalise(self):
+        self.finalising = True
+        for a in self.analysers.values():
+            a.request_stop()
+            log.debug(f"Stop request sent to analyser {a}")
+        for w in self._environment.watchers.values():
+            w.request_stop()
+            log.debug(f"Stop request sent to watcher {w}")
+        rsync_finaliser_threads = []
+        for p in self.rsync_processes.keys():
+            # Collect the running rsyncer finaliser threads to pass to the dormancy checker
+            rsync_finaliser_threads.append(self._finalise_rsyncer(p))
+            log.debug(f"Finalised rsyncer {p}")
+
+        # Run the session cleanup function in a separate thread
+        cleanup_upon_dormancy_thread = threading.Thread(
+            target=self.clean_up_once_dormant,
+            args=[rsync_finaliser_threads],
+            daemon=True,
+        )
+        cleanup_upon_dormancy_thread.start()
+
+    def update_visit_time(self, new_end_time: datetime):
+        # Convert the received server timestamp into the local equivalent
+        self.visit_end_time = new_end_time + self.server_time_offset
+        for rp in self.rsync_processes.values():
+            rp._end_time = self.visit_end_time
+
     def _start_rsyncer_multigrid(
         self,
         source: Path,
@@ -100,10 +232,14 @@ class MultigridController:
         tag: str = "",
         limited: bool = False,
     ):
-        log.info(f"starting multigrid rsyncer: {source}")
+        log.info(f"Starting multigrid rsyncer: {source}")
+        log.debug(f"Analysis of {source} is {('enabled' if analyse else 'disabled')}")
         destination_overrides = destination_overrides or {}
-        machine_data = requests.get(
-            f"{self._environment.url.geturl()}/instruments/{self.instrument_name}/machine"
+        machine_data = capture_get(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=self.instrument_name,
         ).json()
         if destination_overrides.get(source):
             destination = (
@@ -118,7 +254,7 @@ class MultigridController:
                     break
             else:
                 self._environment.default_destinations[source] = (
-                    f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+                    f"{datetime.now().year}"
                 )
                 destination = determine_default_destination(
                     self._environment.visit,
@@ -136,37 +272,65 @@ class MultigridController:
             source,
             destination,
             force_metadata=self.processing_enabled,
-            analyse=not extra_directory and use_suggested_path and analyse,
+            analyse=analyse,
             remove_files=remove_files,
             tag=tag,
             limited=limited,
             transfer=machine_data.get("data_transfer_enabled", True),
             restarted=str(source) in self.rsync_restarts,
         )
-        self.ws.send(json.dumps({"message": "refresh"}))
+        self.ws.send(
+            json.dumps(
+                {
+                    "message": "refresh",
+                    "target": "rsyncer",
+                    "session_id": self.session_id,
+                }
+            )
+        )
 
     def _rsyncer_stopped(self, source: Path, explicit_stop: bool = False):
         if explicit_stop:
-            remove_url = (
-                f"{self.murfey_url}/sessions/{self.session_id}/rsyncer/{str(source)}"
+            capture_delete(
+                base_url=self.murfey_url,
+                router_name="session_control.router",
+                function_name="delete_rsyncer",
+                session_id=self.session_id,
+                data={"path": str(source)},
             )
-            requests.delete(remove_url)
         else:
-            stop_url = f"{self.murfey_url}/sessions/{self.session_id}/rsyncer_stopped"
-            capture_post(stop_url, json={"source": str(source)})
+            capture_post(
+                base_url=self.murfey_url,
+                router_name="session_control.router",
+                function_name="register_stopped_rsyncer",
+                session_id=self.session_id,
+                data={"path": str(source)},
+            )
 
     def _finalise_rsyncer(self, source: Path):
+        """
+        Starts a new Rsyncer thread that cleans up the directories, and returns that
+        thread to be managed by a central thread.
+        """
         finalise_thread = threading.Thread(
             name=f"Controller finaliser thread ({source})",
             target=self.rsync_processes[source].finalise,
+            kwargs={"thread": False},
             daemon=True,
         )
         finalise_thread.start()
+        log.debug(f"Started RSync cleanup for {str(source)}")
+        return finalise_thread
 
     def _restart_rsyncer(self, source: Path):
         self.rsync_processes[source].restart()
-        restarted_url = f"{self.murfey_url}/sessions/{self.session_id}/rsyncer_started"
-        capture_post(restarted_url, json={"source": str(source)})
+        capture_post(
+            base_url=self.murfey_url,
+            router_name="session_control.router",
+            function_name="register_restarted_rsyncer",
+            session_id=self.session_id,
+            data={"path": str(source)},
+        )
 
     def _request_watcher_stop(self, source: Path):
         self._environment.watchers[source]._stopping = True
@@ -188,10 +352,13 @@ class MultigridController:
         log.info(f"starting rsyncer: {source}")
         if transfer:
             # Always make sure the destination directory exists
-            make_directory_url = (
-                f"{self.murfey_url}/sessions/{self.session_id}/make_rsyncer_destination"
+            capture_post(
+                base_url=self.murfey_url,
+                router_name="file_io_instrument.router",
+                function_name="make_rsyncer_destination",
+                session_id=self.session_id,
+                data={"destination": destination},
             )
-            capture_post(make_directory_url, json={"destination": destination})
         if self._environment:
             self._environment.default_destinations[source] = destination
             if self._environment.gain_ref and visit_path:
@@ -199,7 +366,7 @@ class MultigridController:
                 rsync_cmd = [
                     "rsync",
                     f"{posix_path(self._environment.gain_ref)!r}",  # '!r' will print strings in ''
-                    f"{self._environment.url.hostname}::{visit_path}/processing",
+                    f"{self._environment.url.hostname}::{self.rsync_module}/{visit_path}/processing",
                 ]
                 # Wrap in bash shell
                 cmd = [
@@ -217,6 +384,7 @@ class MultigridController:
             self.rsync_processes[source] = RSyncer(
                 source,
                 basepath_remote=Path(destination),
+                rsync_module=self.rsync_module,
                 server_url=(
                     urlparse(self.rsync_url)
                     if self.rsync_url
@@ -225,6 +393,7 @@ class MultigridController:
                 stop_callback=self._rsyncer_stopped,
                 do_transfer=self.do_transfer,
                 remove_files=remove_files,
+                end_time=self.visit_end_time,
             )
 
             def rsync_result(update: RSyncerUpdate):
@@ -258,12 +427,14 @@ class MultigridController:
                 secondary=True,
             )
             if restarted:
-                restarted_url = (
-                    f"{self.murfey_url}/sessions/{self.session_id}/rsyncer_started"
+                capture_post(
+                    base_url=self.murfey_url,
+                    router_name="session_control.router",
+                    function_name="register_restarted_rsyncer",
+                    session_id=self.session_id,
+                    data={"path": str(source)},
                 )
-                capture_post(restarted_url, json={"source": str(source)})
             else:
-                url = f"{str(self._environment.url.geturl())}/sessions/{str(self._environment.murfey_session)}/rsyncer"
                 rsyncer_data = {
                     "source": str(source),
                     "destination": destination,
@@ -271,7 +442,13 @@ class MultigridController:
                     "transferring": self.do_transfer or self._environment.demo,
                     "tag": tag,
                 }
-                requests.post(url, json=rsyncer_data)
+                capture_post(
+                    base_url=self.murfey_url,
+                    router_name="session_control.router",
+                    function_name="register_rsyncer",
+                    session_id=self._environment.murfey_session,
+                    data=rsyncer_data,
+                )
         self._environment.watchers[source] = DirWatcher(source, settling_time=30)
 
         if not self.analysers.get(source) and analyse:
@@ -336,114 +513,80 @@ class MultigridController:
         if self._register_dc and response.get("form"):
             self._form_values = {k: str(v) for k, v in response.get("form", {}).items()}
             log.info(
-                f"gain reference is set to {self._form_values.get('gain_ref')}, {self._environment.data_collection_parameters.get('gain_ref')}"
+                f"gain reference is set to {self._form_values.get('gain_ref')}, {self._environment.gain_ref}"
             )
             if self._form_values.get("gain_ref") in (None, "None"):
-                self._form_values["gain_ref"] = (
-                    self._environment.data_collection_parameters.get("gain_ref")
-                )
-            self._form_dependencies = response.get("dependencies", {})
+                self._form_values["gain_ref"] = self._environment.gain_ref
             self._data_collection_form_complete = True
         elif self._register_dc is None:
             self._data_collection_form_complete = True
 
-    def _start_dc(self, json, from_form: bool = False):
+    def _start_dc(self, metadata_json, from_form: bool = False):
         if self.dummy_dc:
             return
         # for multigrid the analyser sends the message straight to _start_dc by-passing user input
         # it is then necessary to extract the data from the message
         if from_form:
-            json = json.get("form", {})
-            json = {k: str(v) for k, v in json.items()}
-        self._environment.data_collection_parameters = {
-            k: None if v == "None" else v for k, v in json.items()
-        }
-        source = Path(json["source"])
+            metadata_json = metadata_json.get("form", {})
+            # Safely convert all entries into strings, but leave None as-is
+            metadata_json = {
+                k: str(v) if v is not None else None for k, v in metadata_json.items()
+            }
+        self._environment.dose_per_frame = metadata_json.get("dose_per_frame")
+        self._environment.gain_ref = metadata_json.get("gain_ref")
+        self._environment.symmetry = metadata_json.get("symmetry")
+        self._environment.eer_fractionation = metadata_json.get("eer_fractionation")
+        source = Path(metadata_json["source"])
         context = self.analysers[source]._context
+        if context:
+            context.data_collection_parameters = {
+                k: None if v == "None" else v for k, v in metadata_json.items()
+            }
         if isinstance(context, TomographyContext):
-            if from_form:
-                requests.post(
-                    f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/tomography_processing_parameters",
-                    json=json,
-                )
-
-            source = Path(json["source"])
-
-            self._environment.id_tag_registry["data_collection_group"].append(
-                str(source)
+            source = Path(metadata_json["source"])
+            context.register_tomography_data_collections(
+                file_extension=metadata_json["file_extension"],
+                image_directory=str(self._environment.default_destinations[source]),
+                environment=self._environment,
             )
-            url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self.session_id}/register_data_collection_group"
-            dcg_data = {
-                "experiment_type": "tomo",
-                "experiment_type_id": 36,
-                "tag": str(source),
-            }
-            requests.post(url, json=dcg_data)
 
-            data = {
-                "voltage": json["voltage"],
-                "pixel_size_on_image": json["pixel_size_on_image"],
-                "experiment_type": json["experiment_type"],
-                "image_size_x": json["image_size_x"],
-                "image_size_y": json["image_size_y"],
-                "file_extension": json["file_extension"],
-                "acquisition_software": json["acquisition_software"],
-                "image_directory": str(self._environment.default_destinations[source]),
-                "tag": json["tilt_series_tag"],
-                "source": str(source),
-                "magnification": json["magnification"],
-                "total_exposed_dose": json.get("total_exposed_dose"),
-                "c2aperture": json.get("c2aperture"),
-                "exposure_time": json.get("exposure_time"),
-                "slit_width": json.get("slit_width"),
-                "phase_plate": json.get("phase_plate", False),
-            }
-            capture_post(
-                f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self._environment.murfey_session}/start_data_collection",
-                json=data,
-            )
-            for recipe in ("em-tomo-preprocess", "em-tomo-align"):
-                capture_post(
-                    f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self._environment.murfey_session}/register_processing_job",
-                    json={
-                        "tag": json["tilt_series_tag"],
-                        "source": str(source),
-                        "recipe": recipe,
-                    },
-                )
             log.info("Registering tomography processing parameters")
-            if self._environment.data_collection_parameters.get("num_eer_frames"):
-                eer_response = requests.post(
-                    f"{str(self._environment.url.geturl())}/visits/{self._environment.visit}/{self._environment.murfey_session}/eer_fractionation_file",
-                    json={
-                        "num_frames": self._environment.data_collection_parameters[
+            if context.data_collection_parameters.get("num_eer_frames"):
+                eer_response = capture_post(
+                    base_url=str(self._environment.url.geturl()),
+                    router_name="file_io_instrument.router",
+                    function_name="write_eer_fractionation_file",
+                    visit_name=self._environment.visit,
+                    session_id=self._environment.murfey_session,
+                    data={
+                        "num_frames": context.data_collection_parameters[
                             "num_eer_frames"
                         ],
-                        "fractionation": self._environment.data_collection_parameters[
-                            "eer_fractionation"
-                        ],
-                        "dose_per_frame": self._environment.data_collection_parameters[
-                            "dose_per_frame"
-                        ],
+                        "fractionation": self._environment.eer_fractionation,
+                        "dose_per_frame": self._environment.dose_per_frame,
                         "fractionation_file_name": "eer_fractionation_tomo.txt",
                     },
                 )
                 eer_fractionation_file = eer_response.json()["eer_fractionation_file"]
-                json.update({"eer_fractionation_file": eer_fractionation_file})
-            requests.post(
-                f"{self._environment.url.geturl()}/sessions/{self._environment.murfey_session}/tomography_preprocessing_parameters",
-                json=json,
-            )
-            context._flush_data_collections()
-            context._flush_processing_jobs()
+                metadata_json.update({"eer_fractionation_file": eer_fractionation_file})
             capture_post(
-                f"{self._environment.url.geturl()}/visits/{self._environment.visit}/{self._environment.murfey_session}/flush_tomography_processing",
-                json={"rsync_source": str(source)},
+                base_url=str(self._environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_tomo_proc_params",
+                session_id=self._environment.murfey_session,
+                data=metadata_json,
             )
-            log.info("tomography processing flushed")
+            capture_post(
+                base_url=str(self._environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="flush_tomography_processing",
+                visit_name=self._environment.visit,
+                session_id=self._environment.murfey_session,
+                data={"rsync_source": str(source)},
+            )
+            log.info("Tomography processing flushed")
 
-        elif isinstance(context, SPAContext) or isinstance(context, SPAModularContext):
-            url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self.session_id}/register_data_collection_group"
+        elif isinstance(context, SPAModularContext):
             dcg_data = {
                 "experiment_type": "single particle",
                 "experiment_type_id": 37,
@@ -459,31 +602,42 @@ class MultigridController:
                     else None
                 ),
             }
-            capture_post(url, json=dcg_data)
+            capture_post(
+                base_url=str(self._environment.url.geturl()),
+                router_name="workflow.router",
+                function_name="register_dc_group",
+                visit_name=self._environment.visit,
+                session_id=self.session_id,
+                data=dcg_data,
+            )
             if from_form:
                 data = {
-                    "voltage": json["voltage"],
-                    "pixel_size_on_image": json["pixel_size_on_image"],
-                    "experiment_type": json["experiment_type"],
-                    "image_size_x": json["image_size_x"],
-                    "image_size_y": json["image_size_y"],
-                    "file_extension": json["file_extension"],
-                    "acquisition_software": json["acquisition_software"],
+                    "voltage": metadata_json["voltage"],
+                    "pixel_size_on_image": metadata_json["pixel_size_on_image"],
+                    "experiment_type": metadata_json["experiment_type"],
+                    "image_size_x": metadata_json["image_size_x"],
+                    "image_size_y": metadata_json["image_size_y"],
+                    "file_extension": metadata_json["file_extension"],
+                    "acquisition_software": metadata_json["acquisition_software"],
                     "image_directory": str(
                         self._environment.default_destinations[source]
                     ),
                     "tag": str(source),
                     "source": str(source),
-                    "magnification": json["magnification"],
-                    "total_exposed_dose": json.get("total_exposed_dose"),
-                    "c2aperture": json.get("c2aperture"),
-                    "exposure_time": json.get("exposure_time"),
-                    "slit_width": json.get("slit_width"),
-                    "phase_plate": json.get("phase_plate", False),
+                    "magnification": metadata_json["magnification"],
+                    "total_exposed_dose": metadata_json.get("total_exposed_dose"),
+                    "c2aperture": metadata_json.get("c2aperture"),
+                    "exposure_time": metadata_json.get("exposure_time"),
+                    "slit_width": metadata_json.get("slit_width"),
+                    "phase_plate": metadata_json.get("phase_plate", False),
                 }
                 capture_post(
-                    f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self.session_id}/start_data_collection",
-                    json=data,
+                    base_url=str(self._environment.url.geturl()),
+                    router_name="workflow.router",
+                    function_name="start_dc",
+                    visit_name=self._environment.visit,
+                    session_id=self.session_id,
+                    data=data,
                 )
                 for recipe in (
                     "em-spa-preprocess",
@@ -493,57 +647,45 @@ class MultigridController:
                     "em-spa-refine",
                 ):
                     capture_post(
-                        f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self.session_id}/register_processing_job",
-                        json={
+                        base_url=str(self._environment.url.geturl()),
+                        router_name="workflow.router",
+                        function_name="register_proc",
+                        visit_name=self._environment.visit,
+                        session_id=self.session_id,
+                        data={
                             "tag": str(source),
                             "source": str(source),
                             "recipe": recipe,
                         },
                     )
-                log.info(f"Posting SPA processing parameters: {json}")
+                log.info(f"Posting SPA processing parameters: {metadata_json}")
                 response = capture_post(
-                    f"{self._environment.url.geturl()}/sessions/{self.session_id}/spa_processing_parameters",
-                    json={
-                        **{k: None if v == "None" else v for k, v in json.items()},
+                    base_url=str(self._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="register_spa_proc_params",
+                    session_id=self.session_id,
+                    data={
+                        **{
+                            k: None if v == "None" else v
+                            for k, v in metadata_json.items()
+                        },
                         "tag": str(source),
                     },
                 )
                 if response and not str(response.status_code).startswith("2"):
                     log.warning(f"{response.reason}")
                 capture_post(
-                    f"{self._environment.url.geturl()}/visits/{self._environment.visit}/{self.session_id}/flush_spa_processing",
-                    json={"tag": str(source)},
+                    base_url=str(self._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="flush_spa_processing",
+                    visit_name=self._environment.visit,
+                    session_id=self.session_id,
+                    data={"tag": str(source)},
                 )
-            if isinstance(context, SPAContext):
-                url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/{self.session_id}/start_data_collection"
-                self._environment.listeners["data_collection_group_ids"] = {
-                    partial(
-                        context._register_data_collection,
-                        url=url,
-                        data=json,
-                        environment=self._environment,
-                    )
-                }
-                self._environment.listeners["data_collection_ids"] = {
-                    partial(
-                        context._register_processing_job,
-                        parameters=json,
-                        environment=self._environment,
-                    )
-                }
-                url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/spa_processing"
-                self._environment.listeners["processing_job_ids"] = {
-                    partial(
-                        context._launch_spa_pipeline,
-                        url=url,
-                        environment=self._environment,
-                    )
-                }
 
     def _increment_file_count(
         self, observed_files: List[Path], source: str, destination: str
     ):
-        url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/increment_rsync_file_count"
         num_data_files = len(
             [
                 f
@@ -559,7 +701,13 @@ class MultigridController:
             "increment_count": len(observed_files),
             "increment_data_count": num_data_files,
         }
-        requests.post(url, json=data)
+        capture_post(
+            base_url=str(self._environment.url.geturl()),
+            router_name="prometheus.router",
+            function_name="increment_rsync_file_count",
+            visit_name=self._environment.visit,
+            data=data,
+        )
 
     # Prometheus can handle higher traffic so update for every transferred file rather
     # than batching as we do for the Murfey database updates in _increment_transferred_files
@@ -567,7 +715,6 @@ class MultigridController:
         self, update: RSyncerUpdate, source: str, destination: str
     ):
         if update.outcome is TransferResult.SUCCESS:
-            url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/increment_rsync_transferred_files_prometheus"
             data_files = (
                 [update]
                 if update.file_path.suffix in self._data_suffixes
@@ -586,17 +733,38 @@ class MultigridController:
                 "increment_data_count": len(data_files),
                 "data_bytes": sum(f.file_size for f in data_files),
             }
-            requests.post(url, json=data)
+            capture_post(
+                base_url=str(self._environment.url.geturl()),
+                router_name="prometheus.router",
+                function_name="increment_rsync_transferred_files_prometheus",
+                visit_name=self._environment.visit,
+                data=data,
+            )
 
     def _increment_transferred_files(
-        self, updates: List[RSyncerUpdate], source: str, destination: str
+        self,
+        updates: List[RSyncerUpdate],
+        num_skipped_files: int,
+        source: str,
+        destination: str,
     ):
+        capture_post(
+            base_url=str(self._environment.url.geturl()),
+            router_name="prometheus.router",
+            function_name="increment_rsync_skipped_files_prometheus",
+            visit_name=self._environment.visit,
+            data={
+                "source": source,
+                "session_id": self.session_id,
+                "increment_count": num_skipped_files,
+            },
+        )
+
         checked_updates = [
             update for update in updates if update.outcome is TransferResult.SUCCESS
         ]
         if not checked_updates:
             return
-        url = f"{str(self._environment.url.geturl())}/visits/{str(self._environment.visit)}/increment_rsync_transferred_files"
         data_files = [
             u
             for u in updates
@@ -614,4 +782,10 @@ class MultigridController:
             "increment_data_count": len(data_files),
             "data_bytes": sum(f.file_size for f in data_files),
         }
-        requests.post(url, json=data)
+        capture_post(
+            base_url=str(self._environment.url.geturl()),
+            router_name="prometheus.router",
+            function_name="increment_rsync_transferred_files",
+            visit_name=self._environment.visit,
+            data=data,
+        )
