@@ -13,13 +13,14 @@ import queue
 import subprocess
 import threading
 import time
+from datetime import datetime
 from enum import Enum
 from pathlib import Path
-from typing import Callable, List, NamedTuple
+from typing import Awaitable, Callable, List, NamedTuple
 from urllib.parse import ParseResult
 
 from murfey.client.tui.status_bar import StatusBar
-from murfey.util import Observer
+from murfey.util.client import Observer
 
 logger = logging.getLogger("murfey.client.rsync")
 
@@ -54,6 +55,7 @@ class RSyncer(Observer):
         self,
         basepath_local: Path,
         basepath_remote: Path,
+        rsync_module: str,
         server_url: ParseResult,
         stop_callback: Callable = lambda *args, **kwargs: None,
         local: bool = False,
@@ -62,10 +64,12 @@ class RSyncer(Observer):
         remove_files: bool = False,
         required_substrings_for_removal: List[str] = [],
         notify: bool = True,
+        end_time: datetime | None = None,
     ):
         super().__init__()
         self._basepath = basepath_local.absolute()
         self._basepath_remote = basepath_remote
+        self._rsync_module = rsync_module
         self._do_transfer = do_transfer
         self._remove_files = remove_files
         self._required_substrings_for_removal = required_substrings_for_removal
@@ -73,10 +77,20 @@ class RSyncer(Observer):
         self._local = local
         self._server_url = server_url
         self._notify = notify
+        self._finalised = False
+        self._end_time = end_time
+
+        self._skipped_files: List[Path] = []
+
+        # Set rsync destination
         if local:
             self._remote = str(basepath_remote)
         else:
-            self._remote = f"{server_url.hostname}::{basepath_remote}/"
+            self._remote = (
+                f"{server_url.hostname}::{self._rsync_module}/{basepath_remote}/"
+            )
+        logger.debug(f"rsync destination path set to {self._remote}")
+
         # For local tests you can use something along the lines of
         # self._remote = f"wra62962@ws133:/dls/tmp/wra62962/junk/{basepath_remote}"
         # to avoid having to set up an rsync daemon
@@ -95,7 +109,7 @@ class RSyncer(Observer):
         self._statusbar = status_bar
 
     def __repr__(self) -> str:
-        return f"<RSyncer {self._basepath} → {self._remote} ({self.status})"
+        return f"<RSyncer ({self._basepath} → {self._remote}) [{self.status}]"
 
     @classmethod
     def from_rsyncer(cls, rsyncer: RSyncer, **kwargs):
@@ -116,6 +130,7 @@ class RSyncer(Observer):
         return cls(
             rsyncer._basepath,
             rsyncer._basepath_remote,
+            rsyncer._rsync_module,
             rsyncer._server_url,
             local=kwarguments_from_rsyncer["local"],
             status_bar=kwarguments_from_rsyncer["status_bar"],
@@ -172,24 +187,43 @@ class RSyncer(Observer):
             self.thread.join()
         logger.debug("RSync thread stop completed")
 
-    def finalise(self):
+    def request_stop(self):
+        self._stopping = True
+        self._halt_thread = True
+
+    def finalise(
+        self,
+        thread: bool = True,
+        callback: Callable[..., Awaitable[None] | None] | None = None,
+    ):
         self.stop()
         self._remove_files = True
         self._notify = False
-        self.thread = threading.Thread(
-            name=f"RSync finalisation {self._basepath}:{self._remote}",
-            target=self._process,
-            daemon=True,
-        )
-        for f in self._basepath.glob("**/*"):
-            self.queue.put(f)
-        self.stop()
+        if thread:
+            self.thread = threading.Thread(
+                name=f"RSync finalisation {self._basepath}:{self._remote}",
+                target=self._process,
+                daemon=True,
+            )
+            for f in self._basepath.glob("**/*"):
+                self.queue.put(f)
+            self.stop()
+        else:
+            self._transfer(list(self._basepath.glob("**/*")))
+        self._finalised = True
+        if callback:
+            callback()
 
     def enqueue(self, file_path: Path):
         if not self._stopping:
-            absolute_path = (self._basepath / file_path).resolve()
+            absolute_path = self._basepath / file_path
             self.queue.put(absolute_path)
 
+    def flush_skipped(self):
+        for f in self._skipped_files:
+            self.queue.put(f)
+        self._skipped_files = []
+
     def _process(self):
         logger.info("RSync thread starting")
         files_to_transfer: list[Path]
@@ -275,19 +309,30 @@ class RSyncer(Observer):
             self.notify(update)
             updates.append(update)
             time.sleep(0.01)
-            self.notify([update], secondary=True)
+            self.notify([update], num_skipped_files=0, secondary=True)
         # self.notify(updates, secondary=True)
 
         return True
 
-    def _transfer(self, files: list[Path]) -> bool:
+    def _transfer(self, infiles: list[Path]) -> bool:
         """
         Transfer files via an rsync sub-process, and parses the rsync stdout to verify
         the success of the transfer.
         """
 
         # Set up initial variables
-        files = [f for f in files if f.is_file()]
+        if self._end_time:
+            files = [
+                f
+                for f in infiles
+                if f.is_file() and f.stat().st_ctime < self._end_time.timestamp()
+            ]
+            self._skipped_files.extend(set(infiles).difference(set(files)))
+            num_skipped_files = len(set(infiles).difference(set(files)))
+        else:
+            files = [f for f in infiles if f.is_file()]
+            num_skipped_files = 0
+
         previously_transferred = self._files_transferred
         transfer_success: set[Path] = set()
         successful_updates: list[RSyncerUpdate] = []
@@ -485,7 +530,9 @@ class RSyncer(Observer):
             if success:
                 success = result.returncode == 0
 
-        self.notify(successful_updates, secondary=True)
+        self.notify(
+            successful_updates, num_skipped_files=num_skipped_files, secondary=True
+        )
 
         # Print out a summary message for each file transfer batch instead of individual messages
         # List out file paths as stored in memory to see if issue is due to file path mismatch
@@ -517,7 +564,9 @@ class RSyncer(Observer):
             success = False
 
         if result is None:
-            logger.error(f"No rsync process ran for files: {files}")
+            # Only log this as an error if files were scheduled for transfer
+            if files:
+                logger.error(f"No rsync process ran for files: {files}")
         else:
             logger.log(
                 logging.WARNING if result.returncode else logging.DEBUG,
