@@ -9,13 +9,12 @@ from queue import Queue
 from typing import Awaitable, Callable, Dict, List, OrderedDict, TypeVar
 from urllib.parse import urlparse
 
-import requests
 from textual.app import App
 from textual.reactive import reactive
 from textual.widgets import Button, Input
 
 from murfey.client.analyser import Analyser
-from murfey.client.contexts.spa import SPAContext, SPAModularContext
+from murfey.client.contexts.spa import SPAModularContext
 from murfey.client.contexts.tomo import TomographyContext
 from murfey.client.instance_environment import MurfeyInstanceEnvironment
 from murfey.client.rsync import RSyncer, RSyncerUpdate, TransferResult
@@ -33,10 +32,12 @@ from murfey.client.tui.screens import (
 from murfey.client.tui.status_bar import StatusBar
 from murfey.client.watchdir import DirWatcher
 from murfey.client.watchdir_multigrid import MultigridDirWatcher
-from murfey.util import (
+from murfey.util import posix_path
+from murfey.util.client import (
+    capture_delete,
+    capture_get,
     capture_post,
     get_machine_config_client,
-    posix_path,
     read_config,
     set_default_acquisition_output,
 )
@@ -46,10 +47,7 @@ log = logging.getLogger("murfey.tui.app")
 ReactiveType = TypeVar("ReactiveType")
 
 token = read_config()["Murfey"].get("token", "")
-
-requests.get = partial(requests.get, headers={"Authorization": f"Bearer {token}"})
-requests.post = partial(requests.post, headers={"Authorization": f"Bearer {token}"})
-requests.delete = partial(requests.delete, headers={"Authorization": f"Bearer {token}"})
+instrument_name = read_config()["Murfey"].get("instrument_name", "")
 
 
 class MurfeyTUI(App):
@@ -61,7 +59,6 @@ class MurfeyTUI(App):
     rsync_processes: Dict[Path, RSyncer] = {}
     analysers: Dict[Path, Analyser] = {}
     _form_values: dict = reactive({})
-    _form_dependencies: dict = {}
 
     def __init__(
         self,
@@ -82,7 +79,7 @@ class MurfeyTUI(App):
         self._environment = environment or MurfeyInstanceEnvironment(
             urlparse("http://localhost:8000")
         )
-        self._environment.gain_ref = gain_ref
+        self._environment.gain_ref = str(gain_ref)
         self._sources = self._environment.sources or [Path(".")]
         self._url = self._environment.url
         self._default_destinations = self._environment.default_destinations
@@ -154,7 +151,12 @@ class MurfeyTUI(App):
     ):
         log.info(f"starting multigrid rsyncer: {source}")
         destination_overrides = destination_overrides or {}
-        machine_data = requests.get(f"{self._environment.url.geturl()}/machine").json()
+        machine_data = capture_get(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=instrument_name,
+        ).json()
         if destination_overrides.get(source):
             destination = destination_overrides[source] + f"/{extra_directory}"
         else:
@@ -164,7 +166,7 @@ class MurfeyTUI(App):
                     break
             else:
                 self._environment.default_destinations[source] = (
-                    f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+                    f"{datetime.now().year}"
                 )
                 destination = determine_default_destination(
                     self._visit,
@@ -205,16 +207,21 @@ class MurfeyTUI(App):
         log.info(f"starting rsyncer: {source}")
         if transfer:
             # Always make sure the destination directory exists
-            make_directory_url = f"{str(self._url.geturl())}/sessions/{str(self._environment.murfey_session)}/make_rsyncer_destination"
-            capture_post(make_directory_url, json={"destination": destination})
+            capture_post(
+                base_url=str(self._url.geturl()),
+                router_name="file_io_instrument.router",
+                function_name="make_rsyncer_destination",
+                session_id=self._environment.murfey_session,
+                data={"destination": destination},
+            )
         if self._environment:
             self._environment.default_destinations[source] = destination
             if self._environment.gain_ref and visit_path:
                 # Set up rsync command
                 rsync_cmd = [
                     "rsync",
-                    f"{posix_path(self._environment.gain_ref)!r}",
-                    f"{self._url.hostname}::{visit_path}/processing",
+                    f"{posix_path(Path(self._environment.gain_ref))!r}",
+                    f"{self._url.hostname}::{self._machine_config.get('rsync_module', 'data')}/{visit_path}/processing",
                 ]
                 # Encase in bash shell
                 cmd = [
@@ -226,12 +233,13 @@ class MurfeyTUI(App):
                 gain_rsync = subprocess.run(cmd)
                 if gain_rsync.returncode:
                     log.warning(
-                        f"Gain reference file {posix_path(self._environment.gain_ref)!r} was not successfully transferred to {visit_path}/processing"
+                        f"Gain reference file {posix_path(Path(self._environment.gain_ref))!r} was not successfully transferred to {visit_path}/processing"
                     )
         if transfer:
             self.rsync_processes[source] = RSyncer(
                 source,
                 basepath_remote=Path(destination),
+                rsync_module=self._machine_config.get("rsync_module", "data"),
                 server_url=urlparse(rsync_url) if rsync_url else self._url,
                 # local=self._environment.demo,
                 status_bar=self._statusbar,
@@ -270,14 +278,19 @@ class MurfeyTUI(App):
                 ),
                 secondary=True,
             )
-            url = f"{str(self._url.geturl())}/sessions/{str(self._environment.murfey_session)}/rsyncer"
             rsyncer_data = {
                 "source": str(source),
                 "destination": destination,
                 "session_id": self._environment.murfey_session,
                 "transferring": self._do_transfer,
             }
-            requests.post(url, json=rsyncer_data)
+            capture_post(
+                base_url=str(self._url.geturl()),
+                router_name="session_control.router",
+                function_name="register_rsyncer",
+                session_id=self._environment.murfey_session,
+                data=rsyncer_data,
+            )
 
         self._environment.watchers[source] = DirWatcher(source, settling_time=30)
 
@@ -340,7 +353,6 @@ class MurfeyTUI(App):
         self, observed_files: List[Path], source: str, destination: str
     ):
         if len(observed_files):
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/increment_rsync_file_count"
             num_data_files = len(
                 [
                     f
@@ -356,7 +368,13 @@ class MurfeyTUI(App):
                 "increment_count": len(observed_files),
                 "increment_data_count": num_data_files,
             }
-            requests.post(url, json=data)
+            capture_post(
+                base_url=str(self._url.geturl()),
+                router_name="prometheus.router",
+                function_name="increment_rsync_file_count",
+                visit_name=self._visit,
+                data=data,
+            )
 
     # Prometheus can handle higher traffic so update for every transferred file rather
     # than batching as we do for the Murfey database updates in _increment_transferred_files
@@ -364,7 +382,6 @@ class MurfeyTUI(App):
         self, update: RSyncerUpdate, source: str, destination: str
     ):
         if update.outcome is TransferResult.SUCCESS:
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/increment_rsync_transferred_files_prometheus"
             data_files = (
                 [update]
                 if update.file_path.suffix in self._data_suffixes
@@ -383,7 +400,13 @@ class MurfeyTUI(App):
                 "increment_data_count": len(data_files),
                 "data_bytes": sum(f.file_size for f in data_files),
             }
-            requests.post(url, json=data)
+            capture_post(
+                base_url=str(self._url.geturl()),
+                router_name="prometheus.router",
+                function_name="increment_rsync_transferred_files_prometheus",
+                visit_name=self._visit,
+                data=data,
+            )
 
     def _increment_transferred_files(
         self, updates: List[RSyncerUpdate], source: str, destination: str
@@ -393,7 +416,6 @@ class MurfeyTUI(App):
         ]
         if not checked_updates:
             return
-        url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/increment_rsync_transferred_files"
         data_files = [
             u
             for u in updates
@@ -411,7 +433,13 @@ class MurfeyTUI(App):
             "increment_data_count": len(data_files),
             "data_bytes": sum(f.file_size for f in data_files),
         }
-        requests.post(url, json=data)
+        capture_post(
+            base_url=str(self._url.geturl()),
+            router_name="prometheus.router",
+            function_name="increment_rsync_transferred_files",
+            visit_name=self._visit,
+            data=data,
+        )
 
     def _set_register_dc(self, response: str):
         if response == "y":
@@ -437,13 +465,10 @@ class MurfeyTUI(App):
         if self._register_dc and response.get("form"):
             self._form_values = {k: str(v) for k, v in response.get("form", {}).items()}
             log.info(
-                f"gain reference is set to {self._form_values.get('gain_ref')}, {self._environment.data_collection_parameters.get('gain_ref')}"
+                f"gain reference is set to {self._form_values.get('gain_ref')}, {self._environment.gain_ref}"
             )
             if self._form_values.get("gain_ref") in (None, "None"):
-                self._form_values["gain_ref"] = (
-                    self._environment.data_collection_parameters.get("gain_ref")
-                )
-            self._form_dependencies = response.get("dependencies", {})
+                self._form_values["gain_ref"] = self._environment.gain_ref
             self.processing_btn.disabled = False
             self._data_collection_form_complete = True
         elif self._register_dc is None:
@@ -459,101 +484,70 @@ class MurfeyTUI(App):
             )
         )
 
-    def _start_dc(self, json, from_form: bool = False):
+    def _start_dc(self, metadata_json, from_form: bool = False):
         if self._dummy_dc:
             return
         # for multigrid the analyser sends the message straight to _start_dc by-passing user input
         # it is then necessary to extract the data from the message
         if from_form:
-            json = json.get("form", {})
-            json = {k: v if v is None else str(v) for k, v in json.items()}
-        self._environment.data_collection_parameters = {
-            k: None if v == "None" else v for k, v in json.items()
-        }
-        source = Path(json["source"])
+            metadata_json = metadata_json.get("form", {})
+            metadata_json = {
+                k: v if v is None else str(v) for k, v in metadata_json.items()
+            }
+        self._environment.dose_per_frame = metadata_json.get("dose_per_frame")
+        self._environment.gain_ref = metadata_json.get("gain_ref")
+        self._environment.symmetry = metadata_json.get("symmetry")
+        self._environment.eer_fractionation = metadata_json.get("eer_fractionation")
+        source = Path(metadata_json["source"])
         context = self.analysers[source]._context
+        if context:
+            context.data_collection_parameters = {
+                k: None if v == "None" else v for k, v in metadata_json.items()
+            }
         if isinstance(context, TomographyContext):
-            source = Path(json["source"])
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_data_collection_group"
-            dcg_data = {
-                "experiment_type": "tomo",
-                "experiment_type_id": 36,
-                "tag": str(source),
-                "atlas": (
-                    str(self._environment.samples[source].atlas)
-                    if self._environment.samples.get(source)
-                    else ""
-                ),
-                "sample": (
-                    self._environment.samples[source].sample
-                    if self._environment.samples.get(source)
-                    else None
-                ),
-            }
-            capture_post(url, json=dcg_data)
-            data = {
-                "voltage": json["voltage"],
-                "pixel_size_on_image": json["pixel_size_on_image"],
-                "experiment_type": json["experiment_type"],
-                "image_size_x": json["image_size_x"],
-                "image_size_y": json["image_size_y"],
-                "file_extension": json["file_extension"],
-                "acquisition_software": json["acquisition_software"],
-                "image_directory": str(self._environment.default_destinations[source]),
-                "tag": json["tilt_series_tag"],
-                "source": str(source),
-                "magnification": json["magnification"],
-                "total_exposed_dose": json.get("total_exposed_dose"),
-                "c2aperture": json.get("c2aperture"),
-                "exposure_time": json.get("exposure_time"),
-                "slit_width": json.get("slit_width"),
-                "phase_plate": json.get("phase_plate", False),
-            }
-            capture_post(
-                f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection",
-                json=data,
+            source = Path(metadata_json["source"])
+            context.register_tomography_data_collections(
+                file_extension=metadata_json["file_extension"],
+                image_directory=str(self._environment.default_destinations[source]),
+                environment=self._environment,
             )
-            for recipe in ("em-tomo-preprocess", "em-tomo-align"):
-                capture_post(
-                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_processing_job",
-                    json={
-                        "tag": json["tilt_series_tag"],
-                        "source": str(source),
-                        "recipe": recipe,
-                    },
-                )
+
             log.info("Registering tomography processing parameters")
-            if self.app._environment.data_collection_parameters.get("num_eer_frames"):
-                eer_response = requests.post(
-                    f"{str(self.app._environment.url.geturl())}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/eer_fractionation_file",
-                    json={
-                        "num_frames": self.app._environment.data_collection_parameters[
+            if context.data_collection_parameters.get("num_eer_frames"):
+                eer_response = capture_post(
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="file_io_instrument.router",
+                    function_name="write_eer_fractionation_file",
+                    visit_name=self.app._environment.visit,
+                    session_id=self.app._environment.murfey_session,
+                    data={
+                        "num_frames": context.data_collection_parameters[
                             "num_eer_frames"
                         ],
-                        "fractionation": self.app._environment.data_collection_parameters[
-                            "eer_fractionation"
-                        ],
-                        "dose_per_frame": self.app._environment.data_collection_parameters[
-                            "dose_per_frame"
-                        ],
+                        "fractionation": self.app._environment.eer_fractionation,
+                        "dose_per_frame": self.app._environment.dose_per_frame,
                         "fractionation_file_name": "eer_fractionation_tomo.txt",
                     },
                 )
                 eer_fractionation_file = eer_response.json()["eer_fractionation_file"]
-                json.update({"eer_fractionation_file": eer_fractionation_file})
-            requests.post(
-                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/tomography_preprocessing_parameters",
-                json=json,
-            )
-            context._flush_data_collections()
-            context._flush_processing_jobs()
+                metadata_json.update({"eer_fractionation_file": eer_fractionation_file})
             capture_post(
-                f"{self.app._environment.url.geturl()}/visits/{self._visit}/{self.app._environment.murfey_session}/flush_tomography_processing",
-                json={"rsync_source": str(source)},
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="register_tomo_proc_params",
+                session_id=self.app._environment.murfey_session,
+                data=metadata_json,
             )
-            log.info("tomography processing flushed")
-        elif isinstance(context, SPAContext) or isinstance(context, SPAModularContext):
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_data_collection_group"
+            capture_post(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="workflow.tomo_router",
+                function_name="flush_tomography_processing",
+                visit_name=self._visit,
+                session_id=self.app._environment.murfey_session,
+                data={"rsync_source": str(source)},
+            )
+            log.info("Tomography processing flushed")
+        elif isinstance(context, SPAModularContext):
             dcg_data = {
                 "experiment_type": "single particle",
                 "experiment_type_id": 37,
@@ -569,31 +563,42 @@ class MurfeyTUI(App):
                     else None
                 ),
             }
-            capture_post(url, json=dcg_data)
+            capture_post(
+                base_url=str(self._url.geturl()),
+                router_name="workflow.router",
+                function_name="register_dc_group",
+                visit_name=self._visit,
+                session_id=self._environment.murfey_session,
+                data=dcg_data,
+            )
             if from_form:
                 data = {
-                    "voltage": json["voltage"],
-                    "pixel_size_on_image": json["pixel_size_on_image"],
-                    "experiment_type": json["experiment_type"],
-                    "image_size_x": json["image_size_x"],
-                    "image_size_y": json["image_size_y"],
-                    "file_extension": json["file_extension"],
-                    "acquisition_software": json["acquisition_software"],
+                    "voltage": metadata_json["voltage"],
+                    "pixel_size_on_image": metadata_json["pixel_size_on_image"],
+                    "experiment_type": metadata_json["experiment_type"],
+                    "image_size_x": metadata_json["image_size_x"],
+                    "image_size_y": metadata_json["image_size_y"],
+                    "file_extension": metadata_json["file_extension"],
+                    "acquisition_software": metadata_json["acquisition_software"],
                     "image_directory": str(
                         self._environment.default_destinations[source]
                     ),
                     "tag": str(source),
                     "source": str(source),
-                    "magnification": json["magnification"],
-                    "total_exposed_dose": json.get("total_exposed_dose"),
-                    "c2aperture": json.get("c2aperture"),
-                    "exposure_time": json.get("exposure_time"),
-                    "slit_width": json.get("slit_width"),
-                    "phase_plate": json.get("phase_plate", False),
+                    "magnification": metadata_json["magnification"],
+                    "total_exposed_dose": metadata_json.get("total_exposed_dose"),
+                    "c2aperture": metadata_json.get("c2aperture"),
+                    "exposure_time": metadata_json.get("exposure_time"),
+                    "slit_width": metadata_json.get("slit_width"),
+                    "phase_plate": metadata_json.get("phase_plate", False),
                 }
                 capture_post(
-                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection",
-                    json=data,
+                    base_url=str(self._url.geturl()),
+                    router_name="workflow.router",
+                    function_name="start_dc",
+                    visit_name=self._visit,
+                    session_id=self._environment.murfey_session,
+                    data=data,
                 )
                 for recipe in (
                     "em-spa-preprocess",
@@ -603,18 +608,28 @@ class MurfeyTUI(App):
                     "em-spa-refine",
                 ):
                     capture_post(
-                        f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_processing_job",
-                        json={
+                        base_url=str(self._url.geturl()),
+                        router_name="workflow.router",
+                        function_name="register_proc",
+                        visit_name=self._visit,
+                        session_id=self._environment.murfey_session,
+                        data={
                             "tag": str(source),
                             "source": str(source),
                             "recipe": recipe,
                         },
                     )
-                log.info(f"Posting SPA processing parameters: {json}")
+                log.info(f"Posting SPA processing parameters: {metadata_json}")
                 response = capture_post(
-                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/spa_processing_parameters",
-                    json={
-                        **{k: None if v == "None" else v for k, v in json.items()},
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="register_spa_proc_params",
+                    session_id=self.app._environment.murfey_session,
+                    data={
+                        **{
+                            k: None if v == "None" else v
+                            for k, v in metadata_json.items()
+                        },
                         "tag": str(source),
                     },
                 )
@@ -626,34 +641,13 @@ class MurfeyTUI(App):
                 if not str(response.status_code).startswith("2"):
                     log.warning(f"{response.reason}")
                 capture_post(
-                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/flush_spa_processing",
-                    json={"tag": str(source)},
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="flush_spa_processing",
+                    visit_name=self.app._environment.visit,
+                    session_id=self.app._environment.murfey_session,
+                    data={"tag": str(source)},
                 )
-            if isinstance(context, SPAContext):
-                url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection"
-                self._environment.listeners["data_collection_group_ids"] = {
-                    partial(
-                        context._register_data_collection,
-                        url=url,
-                        data=json,
-                        environment=self._environment,
-                    )
-                }
-                self._environment.listeners["data_collection_ids"] = {
-                    partial(
-                        context._register_processing_job,
-                        parameters=json,
-                        environment=self._environment,
-                    )
-                }
-                url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/spa_processing"
-                self._environment.listeners["processing_job_ids"] = {
-                    partial(
-                        context._launch_spa_pipeline,
-                        url=url,
-                        environment=self._environment,
-                    )
-                }
 
     def _set_request_destination(self, response: str):
         if response == "y":
@@ -667,9 +661,7 @@ class MurfeyTUI(App):
         )
 
     def _install_processing_form(self):
-        self.processing_form = ProcessingForm(
-            self._form_values, dependencies=self._form_dependencies
-        )
+        self.processing_form = ProcessingForm(self._form_values)
         self.install_screen(self.processing_form, "processing-form")
 
     def on_input_submitted(self, event: Input.Submitted):
@@ -684,8 +676,10 @@ class MurfeyTUI(App):
             await self.reset()
 
     async def on_mount(self) -> None:
-        exisiting_sessions = requests.get(
-            f"{self._environment.url.geturl()}/sessions"
+        exisiting_sessions = capture_get(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="get_sessions",
         ).json()
         if self.visits:
             self.install_screen(VisitSelection(self.visits), "visit-select-screen")
@@ -712,8 +706,12 @@ class MurfeyTUI(App):
         else:
             session_name = "Client connection"
             resp = capture_post(
-                f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session",
-                json={"session_id": None, "session_name": session_name},
+                base_url=str(self._environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="link_client_to_session",
+                instrument_name=self._environment.instrument_name,
+                client_id=self._environment.client_id,
+                data={"session_id": None, "session_name": session_name},
             )
             if resp:
                 self._environment.murfey_session = resp.json()
@@ -730,8 +728,11 @@ class MurfeyTUI(App):
         if self.rsync_processes and machine_config.get("allow_removal"):
             sources = "\n".join(str(k) for k in self.rsync_processes.keys())
             prompt = f"Remove files from the following:\n {sources} \n"
-            rsync_instances = requests.get(
-                f"{self._environment.url.geturl()}/sessions/{self._environment.murfey_session}/rsyncers"
+            rsync_instances = capture_get(
+                base_url=str(self._environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="get_rsyncers_for_session",
+                session_id=self._environment.murfey_session,
             ).json()
             prompt += f"Copied {sum(r['files_counted'] for r in rsync_instances)} / {sum(r['files_transferred'] for r in rsync_instances)}"
             self.install_screen(
@@ -754,8 +755,11 @@ class MurfeyTUI(App):
         self.exit()
 
     async def action_remove_session(self) -> None:
-        requests.delete(
-            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
+        capture_delete(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="remove_session",
+            session_id=self._environment.murfey_session,
         )
         if self.rsync_processes:
             for rp in self.rsync_processes.values():
@@ -768,8 +772,11 @@ class MurfeyTUI(App):
         self.exit()
 
     def clean_up_quit(self) -> None:
-        requests.delete(
-            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
+        capture_delete(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="remove_session",
+            session_id=self._environment.murfey_session,
         )
         self.exit()
 
@@ -811,11 +818,17 @@ class MurfeyTUI(App):
                 removal_rp.queue.put(f)
             removal_rp.stop()
             log.info(f"rsyncer {rp} rerun with removal")
-        requests.post(
-            f"{self._environment.url.geturl()}/sessions/{self._environment.murfey_session}/successful_processing"
+        capture_post(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="register_processing_success_in_ispyb",
+            session_id=self._environment.murfey_session,
         )
-        requests.delete(
-            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
+        capture_delete(
+            base_url=str(self._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="remove_session",
+            session_id=self._environment.murfey_session,
         )
         self.exit()
 
