@@ -18,7 +18,6 @@ from typing import (
     TypeVar,
 )
 
-import requests
 from pydantic import BaseModel, ValidationError
 from rich.box import SQUARE
 from rich.panel import Panel
@@ -46,8 +45,8 @@ from textual.widgets import (
 )
 from werkzeug.utils import secure_filename
 
-from murfey.client.analyser import Analyser, spa_form_dependencies
-from murfey.client.contexts.spa import SPAContext, SPAModularContext
+from murfey.client.analyser import Analyser
+from murfey.client.contexts.spa import SPAModularContext
 from murfey.client.contexts.tomo import TomographyContext
 from murfey.client.gain_ref import determine_gain_ref
 from murfey.client.instance_environment import (
@@ -55,19 +54,22 @@ from murfey.client.instance_environment import (
     global_env_lock,
 )
 from murfey.client.rsync import RSyncer
-from murfey.client.tui.forms import FormDependency
-from murfey.util import capture_post, get_machine_config_client, posix_path, read_config
-from murfey.util.models import PreprocessingParametersTomo, ProcessingParametersSPA
+from murfey.util import posix_path
+from murfey.util.client import (
+    capture_delete,
+    capture_get,
+    capture_post,
+    get_machine_config_client,
+    read_config,
+)
+from murfey.util.models import ProcessingParametersSPA, ProcessingParametersTomo
 
 log = logging.getLogger("murfey.tui.screens")
 
 ReactiveType = TypeVar("ReactiveType")
 
 token = read_config()["Murfey"].get("token", "")
-
-requests.get = partial(requests.get, headers={"Authorization": f"Bearer {token}"})
-requests.post = partial(requests.post, headers={"Authorization": f"Bearer {token}"})
-requests.delete = partial(requests.delete, headers={"Authorization": f"Bearer {token}"})
+instrument_name = read_config()["Murfey"].get("instrument_name", "")
 
 
 def determine_default_destination(
@@ -81,21 +83,24 @@ def determine_default_destination(
     include_mid_path: bool = True,
     use_suggested_path: bool = True,
 ) -> str:
-    machine_data = requests.get(
-        f"{environment.url.geturl()}/instruments/{environment.instrument_name}/machine"
+    machine_data = capture_get(
+        base_url=str(environment.url.geturl()),
+        router_name="session_control.router",
+        function_name="machine_info_by_instrument",
+        instrument_name=environment.instrument_name,
     ).json()
     _default = ""
     if environment.processing_only_mode and environment.sources:
         log.info(f"Processing only mode with sources {environment.sources}")
-        _default = str(environment.sources[0].resolve()) or str(Path.cwd())
+        _default = str(environment.sources[0].absolute()) or str(Path.cwd())
     elif machine_data.get("data_directories"):
         for data_dir in machine_data["data_directories"]:
-            if source.resolve() == Path(data_dir):
-                _default = destination + f"/{visit}"
+            if source.absolute() == Path(data_dir).absolute():
+                _default = f"{destination}/{visit}"
                 break
             else:
                 try:
-                    mid_path = source.resolve().relative_to(data_dir)
+                    mid_path = source.absolute().relative_to(Path(data_dir).absolute())
                     if use_suggested_path:
                         with global_env_lock:
                             source_name = (
@@ -107,8 +112,12 @@ def determine_default_destination(
                                 _default = environment.destination_registry[source_name]
                             else:
                                 suggested_path_response = capture_post(
-                                    url=f"{str(environment.url.geturl())}/visits/{visit}/{environment.murfey_session}/suggested_path",
-                                    json={
+                                    base_url=str(environment.url.geturl()),
+                                    router_name="file_io_instrument.router",
+                                    function_name="suggest_path",
+                                    visit_name=visit,
+                                    session_id=environment.murfey_session,
+                                    data={
                                         "base_path": f"{destination}/{visit}/{mid_path.parent if include_mid_path else ''}/raw",
                                         "touch": touch,
                                         "extra_directory": extra_directory,
@@ -128,7 +137,7 @@ def determine_default_destination(
         else:
             _default = ""
     else:
-        _default = destination + f"/{visit}"
+        _default = f"{destination}/{visit}"
     return (
         _default + f"/{extra_directory}"
         if not _default.endswith("/")
@@ -194,7 +203,7 @@ def validate_form(form: dict, model: BaseModel) -> bool:
     try:
         convert = lambda x: None if x == "None" else x
         validated = model(**{k: convert(v) for k, v in form.items()})
-        log.info(validated.dict())
+        log.info(validated.model_dump())
         return True
     except (AttributeError, ValidationError) as e:
         log.warning(f"Form validation failed: {str(e)}")
@@ -220,7 +229,7 @@ class _DirectoryTree(DirectoryTree):
                 self.valid_selection = True
                 return
             for d in self._data_directories:
-                if Path(self._selected_path).resolve().is_relative_to(d):
+                if Path(self._selected_path).absolute().is_relative_to(d.absolute()):
                     self.valid_selection = True
                     break
             else:
@@ -257,22 +266,16 @@ class LaunchScreen(Screen):
         super().__init__(*args, **kwargs)
         self._selected_dir = basepath
         self._add_basepath = add_basepath
-        cfg = get_machine_config_client(
-            str(self.app._environment.url.geturl()),
-            instrument_name=self.app._environment.instrument_name,
-            demo=self.app._environment.demo,
-        )
-        self._context: (
-            Type[SPAModularContext] | Type[SPAContext] | Type[TomographyContext]
-        )
-        if cfg.get("modular_spa"):
-            self._context = SPAContext
-        else:
-            self._context = SPAModularContext
+        self._context: Type[SPAModularContext] | Type[TomographyContext]
+        self._context = SPAModularContext
 
     def compose(self):
-        machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine"
+
+        machine_data = capture_get(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=instrument_name,
         ).json()
         self._dir_tree = _DirectoryTree(
             str(self._selected_dir),
@@ -290,7 +293,9 @@ class LaunchScreen(Screen):
         btn_disabled = True
         for d in machine_data.get("data_directories", []):
             if (
-                Path(self._dir_tree._selected_path).resolve().is_relative_to(d)
+                Path(self._dir_tree._selected_path)
+                .absolute()
+                .is_relative_to(Path(d).absolute())
                 or self.app._environment.processing_only_mode
             ):
                 btn_disabled = False
@@ -314,18 +319,13 @@ class LaunchScreen(Screen):
                 self._add_btn.disabled = True
 
     def _add_directory(self, directory: str, add_destination: bool = True):
-        source = Path(self._dir_tree.path).resolve() / directory
+        source = Path(self._dir_tree.path).absolute() / directory
         if add_destination:
             for s in self.app._environment.sources:
                 if source.is_relative_to(s):
                     return
             self.app._environment.sources.append(source)
-            machine_data = requests.get(
-                f"{self.app._environment.url.geturl()}/machine"
-            ).json()
-            self.app._default_destinations[source] = (
-                f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
-            )
+            self.app._default_destinations[source] = f"{datetime.now().year}"
         if self._launch_btn:
             self._launch_btn.disabled = False
         self.query_one("#selected-directories").write(str(source) + "\n")
@@ -355,9 +355,7 @@ class LaunchScreen(Screen):
                     )
                 transfer_routes[s] = _default
             self.app.install_screen(
-                DestinationSelect(
-                    transfer_routes, self._context, dependencies=spa_form_dependencies
-                ),
+                DestinationSelect(transfer_routes, self._context),
                 "destination-select-screen",
             )
             self.app.pop_screen()
@@ -432,13 +430,11 @@ class ProcessingForm(Screen):
         self,
         form: dict,
         *args,
-        dependencies: Dict[str, FormDependency] | None = None,
         **kwargs,
     ):
         super().__init__(*args, **kwargs)
         self._form = form
         self._inputs: Dict[Input, str] = {}
-        self._dependencies = dependencies or {}
 
     def compose(self):
         inputs = []
@@ -456,8 +452,6 @@ class ProcessingForm(Screen):
                 i.value = "None" if default is None else default
             self._inputs[i] = k.name
             inputs.append(i)
-        for i, k in self._inputs.items():
-            self._check_dependency(k, i.value)
         confirm_btn = Button("Confirm", id="confirm-btn")
         if self._form.get("motion_corr_binning") == "2":
             self._vert = VerticalScroll(
@@ -474,7 +468,7 @@ class ProcessingForm(Screen):
     def _write_params(
         self,
         params: dict | None = None,
-        model: PreprocessingParametersTomo | ProcessingParametersSPA | None = None,
+        model: ProcessingParametersTomo | ProcessingParametersSPA | None = None,
     ):
         if params:
             try:
@@ -484,18 +478,28 @@ class ProcessingForm(Screen):
             for k in analyser._context.user_params + analyser._context.metadata_params:
                 self.app.query_one("#info").write(f"{k.label}: {params.get(k.name)}")
             self.app._start_dc(params)
-            if model == PreprocessingParametersTomo:
-                requests.post(
-                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/tomography_preprocessing_parameters",
-                    json=params,
+            if model == ProcessingParametersTomo:
+                capture_post(
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="workflow.tomo_router",
+                    function_name="register_tomo_proc_params",
+                    session_id=self.app._environment.murfey_session,
+                    data=params,
                 )
             elif model == ProcessingParametersSPA:
-                requests.post(
-                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/spa_processing_parameters",
-                    json=params,
+                capture_post(
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="register_spa_proc_params",
+                    session_id=self.app._environment.murfey_session,
+                    data=params,
                 )
-                requests.post(
-                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/flush_spa_processing"
+                capture_post(
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="workflow.spa_router",
+                    function_name="flush_spa_processing",
+                    visit_name=self.app._environment.visit,
+                    session_id=self.app._environment.murfey_session,
                 )
 
     def on_switch_changed(self, event):
@@ -511,23 +515,6 @@ class ProcessingForm(Screen):
         else:
             k = self._inputs[event.switch]
             self._form[k] = event.value
-            self._check_dependency(k, event.value)
-
-    def _check_dependency(self, key: str, value: Any):
-        if x := self._dependencies.get(key):
-            for d, v in x.dependencies.items():
-                if value == x.trigger_value:
-                    self._form[d] = v
-                    for i, dk in self._inputs.items():
-                        if dk == d:
-                            i.value = v
-                            i.disabled = True
-                            break
-                else:
-                    for i, dk in self._inputs.items():
-                        if dk == d:
-                            i.disabled = False
-                            break
 
     def on_input_changed(self, event):
         k = self._inputs[event.input]
@@ -654,15 +641,26 @@ class SessionSelection(Screen):
             session_id = self.app._environment.murfey_session
             self.app.pop_screen()
         session_name = "Client connection"
-        self.app._environment.murfey_session = requests.post(
-            f"{self.app._environment.url.geturl()}/instruments/{self.app._environment.instrument_name}/clients/{self.app._environment.client_id}/session",
-            json={"session_id": session_id, "session_name": session_name},
+        self.app._environment.murfey_session = capture_post(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="link_client_to_session",
+            instrument_name=self.app._environment.instrument_name,
+            client_id=self.app._environment.client_id,
+            data={"session_id": session_id, "session_name": session_name},
         ).json()
 
     def _remove_session(self, session_id: int, **kwargs):
-        requests.delete(f"{self.app._environment.url.geturl()}/sessions/{session_id}")
-        exisiting_sessions = requests.get(
-            f"{self.app._environment.url.geturl()}/sessions"
+        capture_delete(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="remove_session",
+            session_id=session_id,
+        )
+        exisiting_sessions = capture_get(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="get_sessions",
         ).json()
         self.app.uninstall_screen("session-select-screen")
         if exisiting_sessions:
@@ -684,8 +682,12 @@ class SessionSelection(Screen):
         else:
             session_name = "Client connection"
             resp = capture_post(
-                f"{self.app._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self.app._environment.client_id}/session",
-                json={"session_id": None, "session_name": session_name},
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="link_client_to_session",
+                instrument_name=self.app._environment.instrument_name,
+                client_id=self.app._environment.client_id,
+                data={"session_id": None, "session_name": session_name},
             )
             if resp:
                 self.app._environment.murfey_session = resp.json()
@@ -705,22 +707,28 @@ class VisitSelection(SwitchSelection):
         text = str(event.button.label)
         self.app._visit = text
         self.app._environment.visit = text
-        response = requests.post(
-            f"{self.app._environment.url.geturl()}/visits/{text}",
-            json={"id": self.app._environment.client_id},
+        response = capture_post(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="register_client_to_visit",
+            visit_name=text,
+            data={"id": self.app._environment.client_id},
         )
         log.info(f"Posted visit registration: {response.status_code}")
-        machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine"
+        machine_data = capture_get(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=instrument_name,
         ).json()
 
         if self._switch_status:
             self.app.install_screen(
                 DirectorySelection(
                     [
-                        p
-                        for p in machine_data.get("data_directories", [])
-                        if Path(p).exists()
+                        path
+                        for path in machine_data.get("data_directories", [])
+                        if Path(path).exists()
                     ]
                 ),
                 "directory-select",
@@ -744,8 +752,11 @@ class VisitSelection(SwitchSelection):
                 self.app.push_screen("launcher")
 
         if machine_data.get("upstream_data_directories"):
-            upstream_downloads = requests.get(
-                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/upstream_visits"
+            upstream_downloads = capture_get(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.correlative_router",
+                function_name="find_upstream_visits",
+                session_id=self.app._environment.murfey_session,
             ).json()
             self.app.install_screen(
                 UpstreamDownloads(upstream_downloads), "upstream-downloads"
@@ -772,18 +783,28 @@ class VisitCreation(Screen):
         text = str(self.visit_name)
         self.app._visit = text
         self.app._environment.visit = text
-        response = requests.post(
-            f"{self.app._environment.url.geturl()}/visits/{text}",
-            json={"id": self.app._environment.client_id},
+        response = capture_post(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="register_client_to_visit",
+            visit_name=text,
+            data={"id": self.app._environment.client_id},
         )
         log.info(f"Posted visit registration: {response.status_code}")
-        machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine"
+        machine_data = capture_get(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=instrument_name,
         ).json()
 
         self.app.install_screen(
             DirectorySelection(
-                [p for p in machine_data.get("data_directories", []) if p.exists()]
+                [
+                    path
+                    for path in machine_data.get("data_directories", [])
+                    if Path(path).exists()
+                ]
             ),
             "directory-select",
         )
@@ -799,15 +820,14 @@ class VisitCreation(Screen):
             )
             self.app.push_screen("gain-ref-select")
         else:
-            if self._switch_status:
-                self.app.push_screen("directory-select")
-            else:
-                self.app.install_screen(LaunchScreen(basepath=Path("./")), "launcher")
-                self.app.push_screen("launcher")
+            self.app.push_screen("directory-select")
 
         if machine_data.get("upstream_data_directories"):
-            upstream_downloads = requests.get(
-                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/upstream_visits"
+            upstream_downloads = capture_get(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.correlative_router",
+                function_name="find_upstream_visits",
+                session_id=self.app._environment.murfey_session,
             ).json()
             self.app.install_screen(
                 UpstreamDownloads(upstream_downloads), "upstream-downloads"
@@ -828,8 +848,11 @@ class UpstreamDownloads(Screen):
         yield Button("Skip", classes="btn-directory")
 
     def on_button_pressed(self, event: Button.Pressed):
-        machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine"
+        machine_data = capture_get(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="session_control.router",
+            function_name="machine_info_by_instrument",
+            instrument_name=instrument_name,
         ).json()
         if machine_data.get("upstream_data_download_directory"):
             # Create the directory locally to save files to
@@ -839,8 +862,12 @@ class UpstreamDownloads(Screen):
             download_dir.mkdir(exist_ok=True)
 
             # Get the paths to the TIFF files generated previously under the same session ID
-            upstream_tiff_paths_response = requests.get(
-                f"{self.app._environment.url.geturl()}/visits/{event.button.label}/{self.app._environment.murfey_session}/upstream_tiff_paths"
+            upstream_tiff_paths_response = capture_get(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.correlative_router",
+                function_name="gather_upstream_tiffs",
+                visit_name=event.button.label,
+                session_id=self.app._environment.murfey_session,
             )
             upstream_tiff_paths = upstream_tiff_paths_response.json() or []
 
@@ -848,9 +875,13 @@ class UpstreamDownloads(Screen):
             for tp in upstream_tiff_paths:
                 (download_dir / tp).parent.mkdir(exist_ok=True, parents=True)
                 # Write TIFF to the specified file path
-                stream_response = requests.get(
-                    f"{self.app._environment.url.geturl()}/visits/{event.button.label}/{self.app._environment.murfey_session}/upstream_tiff/{tp}",
-                    stream=True,
+                stream_response = capture_get(
+                    base_url=str(self.app._environment.url.geturl()),
+                    router_name="session_control.correlative_router",
+                    function_name="get_tiff",
+                    visit_name=event.button.label,
+                    session_id=self.app._environment.murfey_session,
+                    tiff_path=tp,
                 )
                 # Write the file chunk-by-chunk to avoid hogging memory
                 with open(download_dir / tp, "wb") as utiff:
@@ -894,12 +925,12 @@ class GainReference(Screen):
         else:
             if event.button.id == "suggested-gain-ref":
                 self._dir_tree._gain_reference = self._gain_reference
-            visit_path = f"data/{datetime.now().year}/{self.app._environment.visit}"
+            visit_path = f"{datetime.now().year}/{self.app._environment.visit}"
             # Set up rsync command
             rsync_cmd = [
                 "rsync",
                 f"{posix_path(self._dir_tree._gain_reference)!r}",
-                f"{self.app._environment.url.hostname}::{visit_path}/processing/{secure_filename(self._dir_tree._gain_reference.name)}",
+                f"{self.app._environment.rsync_url or self.app._environment.url.hostname}::{self.app._machine_config.get('rsync_module', 'data')}/{visit_path}/processing/{secure_filename(self._dir_tree._gain_reference.name)}",
             ]
             # Encase in bash shell
             cmd = ["bash", "-c", " ".join(rsync_cmd)]
@@ -912,9 +943,12 @@ class GainReference(Screen):
                     log.warning(
                         f"Gain reference file {posix_path(self._dir_tree._gain_reference)!r} was not successfully transferred to {visit_path}/processing"
                     )
-            process_gain_response = requests.post(
-                url=f"{str(self.app._environment.url.geturl())}/sessions/{self.app._environment.murfey_session}/process_gain",
-                json={
+            process_gain_response = capture_post(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="file_io_instrument.router",
+                function_name="process_gain",
+                session_id=self.app._environment.murfey_session,
+                data={
                     "gain_ref": str(self._dir_tree._gain_reference),
                     "eer": bool(
                         self.app._machine_config.get("external_executables_eer")
@@ -929,12 +963,9 @@ class GainReference(Screen):
                 log.info(
                     f"Gain reference file {process_gain_response.json().get('gain_ref')}"
                 )
-                self.app._environment.data_collection_parameters["gain_ref"] = (
-                    process_gain_response.json().get("gain_ref")
+                self.app._environment.gain_ref = process_gain_response.json().get(
+                    "gain_ref"
                 )
-                self.app._environment.data_collection_parameters[
-                    "gain_ref_superres"
-                ] = process_gain_response.json().get("gain_ref_superres")
         if self._switch_status:
             self.app.push_screen("directory-select")
         else:
@@ -954,7 +985,7 @@ class DirectorySelection(SwitchSelection):
 
     def on_button_pressed(self, event: Button.Pressed):
         self.app._multigrid = self._switch_status
-        visit_dir = Path(str(event.button.label)) / self.app._visit
+        visit_dir = Path(str(event.button.label)).absolute() / self.app._visit
         visit_dir.mkdir(exist_ok=True)
         self.app._set_default_acquisition_directories(visit_dir)
         machine_config = get_machine_config_client(
@@ -962,7 +993,7 @@ class DirectorySelection(SwitchSelection):
             instrument_name=self.app._environment.instrument_name,
             demo=self.app._environment.demo,
         )
-        for dir in machine_config["create_directories"].values():
+        for dir in machine_config["create_directories"]:
             (visit_dir / dir).mkdir(exist_ok=True)
         self.app.install_screen(
             LaunchScreen(basepath=visit_dir, add_basepath=True), "launcher"
@@ -975,9 +1006,8 @@ class DestinationSelect(Screen):
     def __init__(
         self,
         transfer_routes: Dict[Path, str],
-        context: Type[SPAContext] | Type[SPAModularContext] | Type[TomographyContext],
+        context: Type[SPAModularContext] | Type[TomographyContext],
         *args,
-        dependencies: Dict[str, FormDependency] | None = None,
         destination_overrides: Optional[Dict[Path, str]] = None,
         use_transfer_routes: bool = False,
         **kwargs,
@@ -986,7 +1016,6 @@ class DestinationSelect(Screen):
         self._transfer_routes = transfer_routes
         self._destination_overrides: Dict[Path, str] = destination_overrides or {}
         self._user_params: Dict[str, str] = {}
-        self._dependencies = dependencies or {}
         self._inputs: Dict[Input, str] = {}
         self._context = context
         self._use_transfer_routes = use_transfer_routes
@@ -994,9 +1023,7 @@ class DestinationSelect(Screen):
     def compose(self):
         bulk = []
         with RadioSet():
-            yield RadioButton(
-                "SPA", value=self._context in (SPAContext, SPAModularContext)
-            )
+            yield RadioButton("SPA", value=self._context is SPAModularContext)
             yield RadioButton("Tomography", value=self._context is TomographyContext)
         if self.app._multigrid:
             machine_config = get_machine_config_client(
@@ -1020,16 +1047,12 @@ class DestinationSelect(Screen):
                     for d in s.glob("*"):
                         if (
                             d.is_dir()
-                            and d.name
-                            not in machine_config["create_directories"].values()
+                            and d.name not in machine_config["create_directories"]
                         ):
-                            machine_data = requests.get(
-                                f"{self.app._environment.url.geturl()}/machine"
-                            ).json()
                             dest = determine_default_destination(
                                 self.app._visit,
                                 s,
-                                f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}",
+                                f"{datetime.now().year}",
                                 self.app._environment,
                                 self.app.analysers,
                                 touch=True,
@@ -1064,7 +1087,7 @@ class DestinationSelect(Screen):
                 instrument_name=self.app._environment.instrument_name,
             )
             for s, d in self._transfer_routes.items():
-                if Path(d).name not in machine_config["create_directories"].values():
+                if Path(d).name not in machine_config["create_directories"]:
                     bulk.append(Label(f"Copy the source {s} to:"))
                     bulk.append(
                         Input(
@@ -1078,9 +1101,7 @@ class DestinationSelect(Screen):
         if self.app._multigrid and self.app._processing_enabled:
             for k in self._context.user_params:
                 params_bulk.append(Label(k.label))
-                val = self.app._environment.data_collection_parameters.get(
-                    k.name
-                ) or str(k.default)
+                val = getattr(self.app._environment, k.name, str(k.default))
                 self._user_params[k.name] = val
                 if val in ("true", "True", True):
                     i = Switch(value=True, id=k.name, classes="input-destination")
@@ -1107,30 +1128,12 @@ class DestinationSelect(Screen):
                     )
                 )
                 self.app._environment.superres = False
-            for i, k in self._inputs.items():
-                self._check_dependency(k, i.value)
         yield VerticalScroll(
             *params_bulk,
             id="user-params",
         )
         yield Button("Confirm", id="destination-btn")
 
-    def _check_dependency(self, key: str, value: Any):
-        if x := self._dependencies.get(key):
-            for d, v in x.dependencies.items():
-                if value == x.trigger_value:
-                    self._user_params[d] = str(v)
-                    for i, dk in self._inputs.items():
-                        if dk == d:
-                            i.value = v
-                            i.disabled = True
-                            break
-                else:
-                    for i, dk in self._inputs.items():
-                        if dk == d:
-                            i.disabled = False
-                            break
-
     def on_switch_changed(self, event):
         if event.switch.id == "superres-multigrid":
             self.app._environment.superres = event.value
@@ -1138,19 +1141,10 @@ class DestinationSelect(Screen):
             for k in self._context.user_params:
                 if event.switch.id == k.name:
                     self._user_params[k.name] = event.value
-                    self._check_dependency(k.name, event.value)
 
     def on_radio_set_changed(self, event: RadioSet.Changed) -> None:
         if event.index == 0:
-            cfg = get_machine_config_client(
-                str(self.app._environment.url.geturl()),
-                instrument_name=self.app._environment.instrument_name,
-                demo=self.app._environment.demo,
-            )
-            if cfg.get("modular_spa"):
-                self._context = SPAContext
-            else:
-                self._context = SPAModularContext
+            self._context = SPAModularContext
         else:
             self._context = TomographyContext
         self.app.pop_screen()
@@ -1159,7 +1153,6 @@ class DestinationSelect(Screen):
             DestinationSelect(
                 self._transfer_routes,
                 self._context,
-                dependencies=spa_form_dependencies,
                 destination_overrides=self._destination_overrides,
                 use_transfer_routes=True,
             ),
@@ -1181,9 +1174,7 @@ class DestinationSelect(Screen):
     def on_button_pressed(self, event):
         if self.app._multigrid and self.app._processing_enabled:
             if self._context == TomographyContext:
-                valid = validate_form(
-                    self._user_params, PreprocessingParametersTomo.Base
-                )
+                valid = validate_form(self._user_params, ProcessingParametersTomo.Base)
             else:
                 valid = validate_form(self._user_params, ProcessingParametersSPA.Base)
             if not valid:
@@ -1200,7 +1191,7 @@ class DestinationSelect(Screen):
             else:
                 self.app._start_rsyncer(s, d)
         for k, v in self._user_params.items():
-            self.app._environment.data_collection_parameters[k] = v
+            setattr(self.app._environment, k, v)
         self.app.pop_screen()
         self.app.push_screen("main")
 
@@ -1256,11 +1247,17 @@ class WaitingScreen(Screen):
     def file_copied(self, *args, **kwargs):
         self.query_one(ProgressBar).advance(1)
         if self.query_one(ProgressBar).progress == self.query_one(ProgressBar).total:
-            requests.post(
-                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/successful_processing"
+            capture_post(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="register_processing_success_in_ispyb",
+                session_id=self.app._environment.murfey_session,
             )
-            requests.delete(
-                f"{self.app._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self.app._environment.client_id}/session"
+            capture_delete(
+                base_url=str(self.app._environment.url.geturl()),
+                router_name="session_control.router",
+                function_name="remove_session",
+                session_id=self.app._environment.murfey_session,
             )
             self.app.exit()
 
@@ -1277,9 +1274,7 @@ class MainScreen(Screen):
             if len(self.app.visits)
             else [Button("No ongoing visits found")]
         )
-        self.app.processing_form = ProcessingForm(
-            self.app._form_values, dependencies=self.app._form_dependencies
-        )
+        self.app.processing_form = ProcessingForm(self.app._form_values)
         yield Header()
         info_widget = RichLog(id="info", markup=True)
         yield info_widget
@@ -1291,6 +1286,10 @@ class MainScreen(Screen):
         yield Footer()
 
     def on_mount(self, event):
-        requests.post(
-            f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/monitoring/1"
+        capture_post(
+            base_url=str(self.app._environment.url.geturl()),
+            router_name="prometheus.router",
+            function_name="change_monitoring_status",
+            visit_name=self.app._environment.visit,
+            on=1,
         )
