@@ -7,6 +7,11 @@ from urllib.parse import ParseResult
 import requests
 
 import murfey
+from murfey.util.api import url_path_for
+
+# Standardised messages to print upon exit
+UPDATE_SUCCESS = "Murfey has been updated. Please restart Murfey."
+UPDATE_FAILURE = "Error occurred while updating Murfey."
 
 
 def check(api_base: ParseResult, install: bool = True, force: bool = False):
@@ -15,8 +20,10 @@ def check(api_base: ParseResult, install: bool = True, force: bool = False):
     If the version number is outside the allowed range then this can trigger
     an update on the client, and in that case will terminate the process.
     """
+    proxy_path = api_base.path.rstrip("/")
     version_check_url = api_base._replace(
-        path="/version", query=f"client_version={murfey.__version__}"
+        path=f"{proxy_path}{url_path_for('bootstrap.version', 'get_version')}",
+        query=f"client_version={murfey.__version__}",
     )
     server_reply = requests.get(version_check_url.geturl())
     if server_reply.status_code != 200:
@@ -37,21 +44,20 @@ def check(api_base: ParseResult, install: bool = True, force: bool = False):
             )
         result = install_murfey(api_base, versions["server"])
         if result:
-            print("\nMurfey has been updated. Please restart Murfey")
-            exit()
+            exit(UPDATE_SUCCESS)
         else:
-            exit("Error occurred while updating Murfey")
+            exit(UPDATE_FAILURE)
 
     if versions["server"] != murfey.__version__:
         if force:
             result = install_murfey(api_base, versions["server"])
             if result:
-                print("\nMurfey has been updated. Please restart Murfey")
-                exit()
+                exit(UPDATE_SUCCESS)
             else:
-                exit("Error occurred while updating Murfey")
+                exit(UPDATE_FAILURE)
         else:
-            print("An update is available, install with 'murfey update'.")
+            # Allow Murfey to start, but print an update prompt
+            print("An update is available, install with 'murfey.client --update'.")
 
 
 def install_murfey(api_base: ParseResult, version: str) -> bool:
@@ -59,6 +65,7 @@ def install_murfey(api_base: ParseResult, version: str) -> bool:
     Return 'true' on success and 'false' on error."""
 
     assert api_base.hostname is not None
+    proxy_path = api_base.path.rstrip("/")
     result = subprocess.run(
         [
             sys.executable,
@@ -67,7 +74,10 @@ def install_murfey(api_base: ParseResult, version: str) -> bool:
             "--trusted-host",
             api_base.hostname,
             "-i",
-            api_base._replace(path="/pypi", query="").geturl(),
+            api_base._replace(
+                path=f"{proxy_path}{url_path_for('bootstrap.pypi', 'get_pypi_index')}",
+                query="",
+            ).geturl(),
             f"murfey[client]=={version}",
         ]
     )
